@@ -72,12 +72,14 @@ static quote_source quote_sources[NUM_SOURCES] = {
   { TRUE,  TRUE,  N_("(none)"), NULL, NULL },
   { FALSE, TRUE,  N_("-- Single Sources --"), NULL, NULL },
   { FALSE, FALSE, "AEX", "AEX", "aex" },
+  { FALSE, FALSE, "AMFI India", "AMFIINDIA", "amfiindia" },
   { FALSE, FALSE, "ASX", "ASX", "asx" },
   { FALSE, FALSE, "DWS", "DWS", "dwsfunds" },
   { FALSE, FALSE, "Fidelity Direct", "FIDELITY_DIRECT", "fidelity_direct" },
   { FALSE, FALSE, "Motley Fool", "FOOL", "fool" },
   { FALSE, FALSE, "Fund Library", "FUNDLIBRARY", "fundlibrary" },
   { FALSE, FALSE, "TD Waterhouse Canada", "TDWATERHOUSE", "tdwaterhouse" },
+  { FALSE, FALSE, "TD Efunds", "TDEFUNDS", "tdefunds" },
   { FALSE, FALSE, "TIAA-CREF", "TIAACREF", "tiaacref" },
   { FALSE, FALSE, "T. Rowe Price", "TRPRICE_DIRECT", "troweprice_direct" }, /* Not Implemented */
   { FALSE, FALSE, "Trustnet", "TRUSTNET", "trustnet" },
@@ -96,6 +98,7 @@ static quote_source quote_sources[NUM_SOURCES] = {
   { FALSE, FALSE, "Canada Mutual (Fund Library, ...)", "CANADAMUTUAL", "canadamutual" },
   { FALSE, FALSE, "Dutch (AEX, ...)", "DUTCH", "dutch" },
   { FALSE, FALSE, "Europe (Yahoo, ...)", "EUROPE", "europe" },
+  { FALSE, FALSE, "India Mutual (AMFI, ...)", "INDIAMUTUAL", "indiamutual" },
   { FALSE, FALSE, "Fidelity (Fidelity, ...)", "FIDELITY", "fidelity" },
   { FALSE, FALSE, "T. Rowe Price", "TRPRICE", "troweprice" },
   { FALSE, FALSE, "U.K. Unit Trusts", "UKUNITTRUSTS", "uk_unit_trusts" },
