@@ -552,6 +552,7 @@ gboolean gncInvoiceGetActive (GncInvoice *invoice)
   return invoice->active;
 }
 
+
 gnc_numeric gncInvoiceGetToChargeAmount (GncInvoice *invoice)
 {
   if (!invoice) return gnc_numeric_zero();
@@ -691,7 +692,7 @@ gnc_lot_match_owner_payment (GNCLot *lot, gpointer user_data)
 
 Transaction * gncInvoicePostToAccount (GncInvoice *invoice, Account *acc,
 				       Timespec *post_date, Timespec *due_date,
-				       const char * memo)
+				       const char * memo, gboolean accumulatesplits)
 {
   Transaction *txn;
   GNCLot *lot = NULL;
@@ -795,7 +796,25 @@ Transaction * gncInvoicePostToAccount (GncInvoice *invoice, Account *acc,
 		gncEntryGetBillAccount (entry));
     if (this_acc) {
       if (gnc_numeric_check (value) == GNC_ERROR_OK) {
-	splitinfo = gncAccountValueAdd (splitinfo, this_acc, value);
+	if (accumulatesplits) {
+	    gncAccountValueAdd (splitinfo, this_acc, value);
+	} else {
+		Split *split;
+
+		split = xaccMallocSplit (invoice->book);
+		/* set action and memo? */
+
+		xaccSplitSetMemo (split, gncEntryGetDescription (entry));
+		xaccSplitSetAction (split, type);
+
+		xaccSplitSetBaseValue (split, (reverse ? gnc_numeric_neg (value)
+				: value),
+				 invoice->currency);
+       	xaccAccountBeginEdit (this_acc);
+        xaccAccountInsertSplit (this_acc, split);
+        xaccAccountCommitEdit (this_acc);
+        xaccTransAppendSplit (txn, split);
+	}
 
 	/* If there is a credit-card account, and this is a CCard
 	 * payment type, the don't add it to the total, and instead
@@ -1065,6 +1084,7 @@ gncOwnerApplyPayment (GncOwner *owner, Account *posted_acc, Account *xfer_acc,
   Split *split;
   GList *lot_list, *fifo = NULL;
   GNCLot *lot, *prepay_lot = NULL;
+  GncInvoice *invoice;
   const char *name;
   gnc_commodity *commodity;
   gnc_numeric split_amt;
@@ -1163,6 +1183,11 @@ gncOwnerApplyPayment (GncOwner *owner, Account *posted_acc, Account *xfer_acc,
     xaccTransAppendSplit (txn, split);
     gnc_lot_add_split (lot, split);
 
+    /* Now send an event for the invoice so it gets updated as paid */
+    invoice = gncInvoiceGetInvoiceFromLot(lot);
+    if (invoice)
+      gnc_engine_generate_event (&invoice->guid, GNC_EVENT_MODIFY);
+
     if (gnc_numeric_zero_p (amount))
       break;
   }
