@@ -27,16 +27,40 @@ gnc_dialog_date_close_parented (GtkWidget *parent, const char *message,
  */
 
 gboolean
-gnc_dialog_dates_acct_parented (GtkWidget *parent, const char *message,
+gnc_dialog_dates_acct_question_parented (GtkWidget *parent, const char *message,
 				const char *ddue_label_message,
 				const char *post_label_message,
 				const char *acct_label_message,
+				const char *question_check_message,
 				gboolean ok_is_default,
 				GList * acct_types, GNCBook *book,
 				GncBillTerm *terms,
 				/* Returned Data... */
 				Timespec *ddue, Timespec *post,
-				char **memo, Account **acct);
+				char **memo, Account **acct, gboolean *answer);
+
+#define gnc_dialog_dates_acct_parented(parent, message,	\
+				ddue_label_message,							\
+				post_label_message,							\
+				acct_label_message,							\
+				ok_is_default,								\
+				acct_types, book,							\
+				terms,										\
+				/* Returned Data... */						\
+				ddue, post,									\
+				memo, acct)									\
+		gnc_dialog_dates_acct_question_parented (parent, message,	\
+				ddue_label_message,							\
+				post_label_message,							\
+				acct_label_message,							\
+				NULL,						\
+				ok_is_default,								\
+				acct_types, book,							\
+				terms,										\
+				/* Returned Data... */						\
+				ddue, post,									\
+				memo, acct, NULL)									\
+
 
 gboolean
 gnc_dialog_date_acct_parented (GtkWidget *parent, const char *message,
