@@ -21,6 +21,8 @@
  *                                                                  *
 \********************************************************************/
 
+#include "config.h"
+#include <stdarg.h>
 #include <glib.h>
 
 #include "Account.h"
@@ -28,10 +30,12 @@
 #include "BackendP.h"
 #include "Group.h"
 #include "GroupP.h"
+#include "gnc-book-p.h"
 #include "gnc-engine-util.h"
 #include "gnc-pricedb.h"
 #include "gnc-pricedb-p.h"
 #include "TransactionP.h"
+#include "messages.h"
 
 /* static short module = MOD_ENGINE; */
 
@@ -61,6 +65,49 @@ xaccBackendGetError (Backend *be)
    return err;
 }
 
+void
+xaccBackendSetMessage (Backend *be, const char *format, ...) {
+   va_list args;
+   char * buffer;
+   
+   if (!be) return;
+  
+   /* If there's already something here, free it */
+   if (be->error_msg) g_free(be->error_msg);
+
+   if (!format) {
+       be->error_msg = NULL;
+       return;
+   }
+
+   va_start(args, format);
+   buffer = (char *)g_strdup_vprintf(format, args);
+   va_end(args);
+
+   be->error_msg = buffer;
+}
+
+/* This should always return a valid char * */
+char *
+xaccBackendGetMessage (Backend *be) {
+   char * msg;
+   
+   if (!be) return g_strdup("ERR_BACKEND_NO_BACKEND");
+   if (!be->error_msg) return NULL;
+
+   /* 
+    * Just return the contents of the error_msg and then set it to
+    * NULL.  This is necessary, because the Backends don't seem to
+    * have a destroy_backend function to take care if freeing stuff
+    * up.  The calling function should free the copy.
+    * Also, this is consistent with the xaccBackendGetError() popping.
+    */
+
+   msg = be->error_msg;
+   be->error_msg = NULL;
+   return msg;
+}
+
 
 /********************************************************************\
  * Fetch the backend                                                *
@@ -69,108 +116,63 @@ xaccBackendGetError (Backend *be)
 Backend *
 xaccAccountGetBackend (Account * acc)
 {
-  Account *parent_acc;
-  AccountGroup * grp;
-
-  if (!acc) return NULL;
-
-  /* find the first account group that has a backend */
-  grp = acc->parent;
-  while (grp) {
-    if (grp->backend) return (grp->backend);
-    parent_acc = grp -> parent;
-    grp = NULL;
-    if (parent_acc) {
-       grp = parent_acc->parent;
-    }
-  }
-  return NULL;
-}
-
-/********************************************************************\
- * Fetch the backend                                                *
-\********************************************************************/
-
-Backend *
-xaccTransactionGetBackend (Transaction *trans)
-{
-  GList *snode, *node;
-  Split *s=NULL;
-
-  if (!trans) return NULL;
-
-  /* find an account */
-  snode = xaccTransGetSplitList(trans);
-  for (node = snode; node; node=node->next)
-  {
-    s = node->data;
-    if (xaccSplitGetAccount(s)) break;
-    s = NULL;
-  }
-
-  /* if transaction is being deleted, it won't have any splits
-   * so lets take a look at the 'original' transaction */
-  if (!s)
-  {
-    snode = xaccTransGetSplitList(trans->orig);
-    for (node = snode; node; node=node->next)
-    {
-      s = node->data;
-      if (xaccSplitGetAccount(s)) break;
-      s = NULL;
-    }
-  }
-  if (!s) return NULL;
-  
-  /* I suppose it would be more 'technically correct' to make sure that
-   * all splits share the same backend, and flag an error if they
-   * don't.  However, at this point, it seems quite unlikely, so we'll
-   * just use the first backend we find.
-   */
-  return xaccAccountGetBackend (xaccSplitGetAccount(s));
-}
-
-/********************************************************************\
- * Set the backend                                                  *
-\********************************************************************/
-
-void 
-xaccGroupSetBackend (AccountGroup *grp, Backend *be)
-{
-   if (!grp) return;
-   grp->backend = be;
+  if (!acc || !acc->book) return NULL;
+  return acc->book->backend;
 }
 
 Backend *
 xaccGroupGetBackend (AccountGroup *grp)
 {
-   while (grp)
-   {
-     Account *parent;
-     if (grp->backend) return (grp->backend);
-     parent = grp->parent;
-     if (!parent) return NULL;
-     grp = parent->parent;
-   }
-   return NULL;
-}
-
-/********************************************************************\
- * Set the backend                                                  *
-\********************************************************************/
-
-void
-xaccPriceDBSetBackend (GNCPriceDB *prdb, Backend *be)
-{
-  if (!prdb) return;
-  prdb->backend = be;
+  grp = xaccGroupGetRoot (grp);
+  if (!grp || !grp->book) return NULL;
+  return grp->book->backend;
 }
 
 Backend *
 xaccPriceDBGetBackend (GNCPriceDB *prdb)
 {
-  if (!prdb) return NULL;
-  return prdb->backend;
+  if (!prdb || !prdb->book) return NULL;
+  return prdb->book->backend;
+}
+
+Backend *
+xaccTransactionGetBackend (Transaction *trans)
+{
+  if (!trans || !trans->book) return NULL;
+  return trans->book->backend;
+}
+
+/***********************************************************************/
+/* Get a clean backend */
+void
+xaccInitBackend(Backend *be)
+{
+    be->session_begin = NULL;
+    be->session_end = NULL;
+    be->destroy_backend = NULL;
+
+    be->load = NULL;
+
+    be->begin = NULL;
+    be->commit = NULL;
+    be->rollback = NULL;
+
+    be->compile_query = NULL;
+    be->free_query = NULL;
+    be->run_query = NULL;
+    be->price_lookup = NULL;
+
+    be->sync = NULL;
+
+    be->events_pending = NULL;
+    be->process_events = NULL;
+
+    be->last_err = ERR_BACKEND_NO_ERR;
+    if (be->error_msg) g_free (be->error_msg);
+    be->error_msg = NULL;
+    be->percentage = NULL;
+
+    be->export = NULL;
 }
 
 /************************* END OF FILE ********************************/
