@@ -34,8 +34,8 @@
  * the GUI & other front-end users.  This file defines these errors.
  */
 
-#ifndef __XACC_BACKEND_H__
-#define __XACC_BACKEND_H__
+#ifndef XACC_BACKEND_H
+#define XACC_BACKEND_H
 
 #include "config.h"
 
@@ -49,6 +49,7 @@ typedef enum {
   ERR_BACKEND_CANT_CONNECT, /* bad dbname/login/passwd or network failure */
   ERR_BACKEND_CONN_LOST,    /* Lost connection to server */
   ERR_BACKEND_LOCKED,       /* in use by another user (ETXTBSY) */
+  ERR_BACKEND_READONLY,	    /* cannot write to file/directory */
   ERR_BACKEND_TOO_NEW,      /* file/db version newer than what we can read */
   ERR_BACKEND_DATA_CORRUPT, /* data in db is corrupt */
   ERR_BACKEND_SERVER_ERR,   /* error in response from server */
@@ -67,7 +68,9 @@ typedef enum {
   ERR_FILEIO_FILE_LOCKERR,   /* mangled locks (unspecified error) */
   ERR_FILEIO_FILE_NOT_FOUND, /* not found / no such file */
   ERR_FILEIO_FILE_TOO_OLD,   /* file version so old we can't read it */
-  ERR_FILEIO_UNKNOWN_FILE_TYPE,
+  ERR_FILEIO_UNKNOWN_FILE_TYPE, /* didn't recognize the file type */
+  ERR_FILEIO_PARSE_ERROR,    /* couldn't parse the data in the file */
+  ERR_FILEIO_BACKUP_ERROR,   /* couldn't make a backup of the file */
   
   /* network errors */
   ERR_NETIO_SHORT_READ = 2000,  /* not enough bytes received */
@@ -91,4 +94,8 @@ typedef enum {
 } GNCBackendError;
 /* NOTE: if you modify GNCBackendError, please update src/scm/gnc.gwp */
 
-#endif /* __XACC_BACKEND_H__ */
+typedef struct backend_s Backend;
+
+typedef void (*GNCBePercentageFunc) (const char *message, double percent);
+
+#endif /* XACC_BACKEND_H */
