@@ -1,7 +1,7 @@
 /********************************************************************\
  * Group.c -- chart of accounts (hierarchical tree of accounts)     *
  * Copyright (C) 1997 Robin D. Clark                                *
- * Copyright (C) 1997-2000 Linas Vepstas <linas@linas.org>          *
+ * Copyright (C) 1997-2001 Linas Vepstas <linas@linas.org>          *
  *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
@@ -24,7 +24,7 @@
 
 #include "config.h"
 
-#include <assert.h>
+#include <glib.h>
 #include <string.h>
 
 #include "Account.h"
@@ -34,7 +34,7 @@
 #include "Group.h"
 #include "GroupP.h"
 #include "TransactionP.h"
-#include "gnc-common.h"
+#include "gnc-book-p.h"
 #include "gnc-engine-util.h"
 #include "gnc-event-p.h"
 #include "gnc-numeric.h"
@@ -52,27 +52,28 @@ static short module = MOD_ENGINE;
 \********************************************************************/
 
 static void
-xaccInitializeAccountGroup (AccountGroup *grp)
+xaccInitializeAccountGroup (AccountGroup *grp, GNCBook *book)
 {
   grp->saved       = 1;
 
   grp->parent      = NULL;
   grp->accounts    = NULL;
 
-  grp->balance     = gnc_numeric_zero();
-
-  grp->backend     = NULL;
+  grp->book        = book;
+  grp->editlevel   = 0;
 }
 
 /********************************************************************\
 \********************************************************************/
 
 AccountGroup *
-xaccMallocAccountGroup (void)
+xaccMallocAccountGroup (GNCBook *book)
 {
-  AccountGroup *grp = g_new (AccountGroup, 1);
+  AccountGroup *grp;
+  g_return_val_if_fail (book, NULL);
 
-  xaccInitializeAccountGroup (grp);
+  grp = g_new (AccountGroup, 1);
+  xaccInitializeAccountGroup (grp, book);
 
   return grp;
 }
@@ -89,41 +90,63 @@ xaccGroupEqual(AccountGroup *ga,
   GList *nb;
 
   if (!ga && !gb) return(TRUE);
-  if (!ga) return(FALSE);
-  if (!gb) return(FALSE);
+
+  if (!ga || !gb)
+  {
+    PWARN ("one is NULL");
+    return(FALSE);
+  }
 
   na = ga->accounts;
   nb = gb->accounts;
 
-  if (!na && nb) return(FALSE);
-  if (na && !nb) return(FALSE);
+  if ((!na && nb) || (na && !nb))
+  {
+    PWARN ("only one has accounts");
+    return(FALSE);
+  }
 
   while (na && nb)
   {
     Account *aa = na->data;
     Account *ab = nb->data;
 
-    if (!xaccAccountEqual(aa, ab, check_guids)) return(FALSE);
+    if (!xaccAccountEqual(aa, ab, check_guids))
+    {
+      char sa[GUID_ENCODING_LENGTH + 1];
+      char sb[GUID_ENCODING_LENGTH + 1];
+
+      guid_to_string_buff (xaccAccountGetGUID (aa), sa);
+      guid_to_string_buff (xaccAccountGetGUID (ab), sb);
+
+      PWARN ("accounts %s and %s differ", sa, sb);
+
+      return(FALSE);
+    }
 
     na = na->next;
     nb = nb->next;
   }
 
-  if (na) return(FALSE);
-  if (nb) return(FALSE);
+  if (na || nb)
+  {
+    PWARN ("different numbers of accounts");
+    return(FALSE);
+  }
 
   return(TRUE);
-} 
+}
 
 /********************************************************************\
 \********************************************************************/
 
-static void
+void
 xaccAccountGroupBeginEdit (AccountGroup *grp)
 {
   GList *node;
 
   if (!grp) return;
+  grp->editlevel++;
 
   for (node = grp->accounts; node; node = node->next)
   {
@@ -148,15 +171,16 @@ xaccAccountGroupCommitEdit (AccountGroup *grp)
   {
     Account *account = node->data;
 
-    xaccAccountCommitEdit (account);
     xaccAccountGroupCommitEdit (account->children);
+    xaccAccountCommitEdit (account);
   }
+  grp->editlevel--;
 }
 
 /********************************************************************\
 \********************************************************************/
 
-void
+static void
 xaccGroupMarkDoFree (AccountGroup *grp)
 {
   GList *node;
@@ -166,13 +190,28 @@ xaccGroupMarkDoFree (AccountGroup *grp)
   for (node = grp->accounts; node; node = node->next)
   {
     Account *account = node->data;
-
     account->do_free = TRUE;
-
     xaccGroupMarkDoFree (account->children); 
   }
 }
 
+void
+xaccAccountGroupDestroy (AccountGroup *grp)
+{
+   xaccGroupMarkDoFree (grp);
+   xaccFreeAccountGroup (grp);
+}
+
+/********************************************************************\
+\********************************************************************/
+
+GNCBook *
+xaccGroupGetBook (AccountGroup *group)
+{
+  if (!group) return NULL;
+  return group->book;
+}
+
 /********************************************************************\
 \********************************************************************/
 
@@ -199,11 +238,18 @@ xaccFreeAccountGroup (AccountGroup *grp)
     while (grp->accounts->next)
     {
       account = grp->accounts->next->data;
-      xaccAccountBeginEdit (account);
+
+      /* FIXME: this and the same code below is kind of hacky.
+       *        actually, all this code seems to assume that
+       *        the account edit levels are all 1. */
+      if (account->editlevel == 0)
+        xaccAccountBeginEdit (account);
+
       xaccAccountDestroy (account);
     }
     account = grp->accounts->data;
-    xaccAccountBeginEdit (account);
+    if (account->editlevel == 0)
+      xaccAccountBeginEdit (account);
     xaccAccountDestroy (account);
 
     if (!root_grp) return;
@@ -212,7 +258,7 @@ xaccFreeAccountGroup (AccountGroup *grp)
   if (grp->parent) grp->parent->children = NULL;
 
   grp->parent   = NULL;
-  grp->balance  = gnc_numeric_zero();
+
   g_free (grp);
 }
 
@@ -315,7 +361,7 @@ xaccPrependAccounts (AccountGroup *grp, GList **accounts_p)
   }
 }
 
-GList *
+AccountList *
 xaccGroupGetSubAccounts (AccountGroup *grp)
 {
   GList *accounts = NULL;
@@ -327,7 +373,7 @@ xaccGroupGetSubAccounts (AccountGroup *grp)
   return g_list_reverse (accounts);
 }
 
-GList *
+AccountList *
 xaccGroupGetAccountList (AccountGroup *grp)
 {
   if (!grp) return NULL;
@@ -340,16 +386,11 @@ xaccGroupGetAccountList (AccountGroup *grp)
 \********************************************************************/
 
 AccountGroup *
-xaccGetAccountRoot (Account * acc) 
+xaccGroupGetRoot (AccountGroup * grp) 
 {
-  AccountGroup * grp;
   AccountGroup * root = NULL;
 
-  if (!acc) return NULL;
-
   /* find the root of the account group structure */
-  grp = acc->parent;
-
   while (grp)
   {
     Account *parent_acc;
@@ -366,6 +407,13 @@ xaccGetAccountRoot (Account * acc)
   return root;
 }
 
+AccountGroup *
+xaccAccountGetRoot (Account * acc) 
+{
+  if (!acc) return NULL;
+  return xaccGroupGetRoot (acc->parent);
+}
+
 /********************************************************************\
  * Fetch an account, given its name                                *
 \********************************************************************/
@@ -495,7 +543,7 @@ xaccGetPeerAccountFromName (Account *acc, const char * name)
   if (!name) return NULL;
 
   /* first, find the root of the account group structure */
-  root = xaccGetAccountRoot (acc);
+  root = xaccAccountGetRoot (acc);
 
   /* now search all accounts hanging off the root */
   peer_acc = xaccGetAccountFromName (root, name);
@@ -518,7 +566,7 @@ xaccGetPeerAccountFromFullName (Account *acc, const char * name,
   if (!name) return NULL;
 
   /* first, find the root of the account group structure */
-  root = xaccGetAccountRoot (acc);
+  root = xaccAccountGetRoot (acc);
 
   /* now search all acounts hanging off the root */
   peer_acc = xaccGetAccountFromFullName (root, name, separator);
@@ -559,7 +607,8 @@ void
 xaccGroupRemoveAccount (AccountGroup *grp, Account *acc)
 {
   if (!acc) return;
-  /* this routine might be called on accounts which 
+
+  /* Note this routine might be called on accounts which 
    * are not yet parented. */
   if (!grp) return;
 
@@ -596,7 +645,7 @@ xaccAccountInsertSubAccount (Account *adult, Account *child)
 
   /* if a container for the children doesn't yet exist, add it */
   if (adult->children == NULL)
-    adult->children = xaccMallocAccountGroup ();
+    adult->children = xaccMallocAccountGroup (adult->book);
 
   /* set back-pointer to parent */
   adult->children->parent = adult;
@@ -625,7 +674,7 @@ group_sort_helper (gconstpointer a, gconstpointer b)
 void
 xaccGroupInsertAccount (AccountGroup *grp, Account *acc)
 {
-  if (!grp) return;
+  if (!grp || !grp->book) return;
   if (!acc) return;
 
   /* If the account is currently in another group, remove it there
@@ -638,14 +687,45 @@ xaccGroupInsertAccount (AccountGroup *grp, Account *acc)
   }
   else
   {
-    if (acc->parent)
+    xaccAccountBeginEdit (acc);
+
+    if (acc->parent) 
+    {
       xaccGroupRemoveAccount (acc->parent, acc);
 
+      /* switch over between books, if needed */
+      if (grp->book != acc->book)
+      {
+// xxxxxxxxxxxxxxxxxxxxxxx
+         /* hack alert -- this implementation is not exactly correct.
+          * If the entity tables are not identical, then the 'from' book 
+          * may have a different backend than the 'to' book.  This means
+          * that we should get the 'from' backend to destroy this account,
+          * and the 'to' backend to save it.  Right now, this is broken.
+          *  
+          * A 'correct' implementation similar to this is in Period.c
+          * except its for transactions ...
+          *
+          * Note also, we need to reparent the children to the new book as well.
+          */
+         PWARN ("reparenting accounts accross books is not correctly supported\n");
+
+         gnc_engine_generate_event (&acc->guid, GNC_EVENT_DESTROY);
+         xaccRemoveEntity (acc->book->entity_table, &acc->guid);
+
+         xaccStoreEntity (grp->book->entity_table, acc, &acc->guid, GNC_ID_ACCOUNT);
+         gnc_engine_generate_event (&acc->guid, GNC_EVENT_CREATE);
+      }
+    }
+
     /* set back-pointer to the account's parent */
     acc->parent = grp;
 
     grp->accounts = g_list_insert_sorted (grp->accounts, acc,
                                           group_sort_helper);
+
+    acc->core_dirty = TRUE;
+    xaccAccountCommitEdit (acc);
   }
 
   grp->saved = 0;
@@ -653,205 +733,6 @@ xaccGroupInsertAccount (AccountGroup *grp, Account *acc)
   gnc_engine_generate_event (&acc->guid, GNC_EVENT_MODIFY);
 }
 
-/********************************************************************\
- * FIXME : this code needs to work differently. 
-\********************************************************************/
-
-void
-xaccRecomputeGroupBalance (AccountGroup *grp)
-{
-  const gnc_commodity * default_currency;
-  Account *account;
-  GList *node;
-
-  if (!grp) return;
-  if (!grp->accounts) return;
-
-  account = grp->accounts->data;
-
-  default_currency = xaccAccountGetCurrency (account);
-
-  grp->balance = gnc_numeric_zero();
-
-  for (node = grp->accounts; node; node = node->next)
-  {
-    Account *account = node->data;
-
-    /* first, get subtotals recursively */
-    if (account->children)
-    {
-      xaccRecomputeGroupBalance (account->children);
-      
-      if (gnc_commodity_equiv (default_currency, account->currency))
-        grp->balance = 
-          gnc_numeric_add (grp->balance, account->children->balance,
-                           GNC_DENOM_AUTO, GNC_DENOM_LCD | GNC_RND_NEVER);
-    }
-
-    /* then add up accounts in this group */
-    xaccAccountRecomputeBalance (account);
-
-    if (gnc_commodity_equiv (default_currency, account->currency))
-      grp->balance = 
-        gnc_numeric_add (grp->balance, account->balance,
-                         GNC_DENOM_AUTO, GNC_DENOM_LCD | GNC_RND_NEVER);
-  }
-}
-
-/********************************************************************\
-\********************************************************************/
-/* account codes will be assigned base-36, with three digits */
-
-#define BASE 36
-
-char *
-xaccGroupGetNextFreeCode (AccountGroup *grp, int digits)
-{
-  GList *node;
-  Account *acc;
-  int maxcode = 0;
-  int i;
-
-  if (!grp) return NULL;
-
-  /* count levels to top */
-  acc = grp->parent;
-  while (acc)
-  {
-    digits --;
-    assert (acc->parent);
-    acc = acc->parent->parent;
-  }
-
-  /* if (0>digits)  we could insert a decimal place, but I am too lazy
-   * to write this code.  It doesn't seem important at the moment ... */
-
-  /* find the largest used code */
-  acc = grp->parent;
-  if (acc && acc->accountCode)
-    maxcode = strtol (acc->accountCode, NULL, BASE);
-
-  for (node = grp->accounts; node; node = node->next)
-  {
-    Account *account = node->data;
-
-    if (account->accountCode)
-    {
-      int code = strtol (account->accountCode, NULL, BASE);
-      if (code > maxcode) maxcode = code;
-    }
-  }
-
-  /* right-shift */
-  for (i = 1; i < digits; i++)
-    maxcode /= BASE;
-
-  maxcode++;
-
-  /* left-shift */
-  for (i = 1; i < digits; i++)
-    maxcode *= BASE;
-
-  /* print */
-  return ultostr ((unsigned long) maxcode, BASE);
-}
-
-/********************************************************************\
-\********************************************************************/
-/* almost identical code to above, but altered to deal with 
- * specified account */
-
-char *
-xaccAccountGetNextChildCode (Account *parent_acc, int digits)
-{
-  GList *node;
-  Account *acc;
-  int maxcode = 0;
-  AccountGroup *grp;
-  int i;
-
-  if (!parent_acc) return NULL;
-
-  /* count levels to top */
-  acc = parent_acc;
-  while (acc)
-  {
-    digits --;
-    assert (acc->parent);   /* all acounts must be in a group */
-    acc = acc->parent->parent;
-  }
-
-  /* if (0>digits)  we could insert a decimal place, but I am too lazy
-   * to write this code.  It doesn't seem important at the moment ... */
-
-  /* find the largest used code */
-  acc = parent_acc;
-  if (acc && acc->accountCode)
-    maxcode = strtol (acc->accountCode, NULL, BASE);
-
-  grp = parent_acc->children;
-  if (grp)
-  {
-    for (node = grp->accounts; node; node = node->next)
-    {
-      Account *account = node->data;
-
-      if (account->accountCode)
-      {
-        int code = strtol (account->accountCode, NULL, BASE);
-        if (code > maxcode) maxcode = code;
-      }
-    }
-  }
-
-  /* right-shift */
-  for (i = 1; i < digits; i++)
-    maxcode /= BASE;
-
-  maxcode++;
-
-  /* left-shift */
-  for (i = 1; i < digits; i++)
-    maxcode *= BASE;
-
-  /* print */
-  return ultostr ((unsigned long) maxcode, BASE);
-}
-
-/********************************************************************\
-\********************************************************************/
-
-void
-xaccGroupDepthAutoCode (AccountGroup *grp)
-{
-   int depth;
-
-   if (!grp) return;
-
-   /* get the depth */
-   depth = xaccGroupGetDepth (grp);
-
-   if (depth < 3) depth = 3;
-
-   xaccGroupAutoCode (grp, depth);
-} 
-
-void
-xaccGroupAutoCode (AccountGroup *grp, int depth)
-{
-  GList *node;
-
-  if (!grp || (depth < 0)) return;
-
-  for (node = grp->accounts; node; node = node->next)
-  {
-    Account *account = node->data;
-
-    xaccAccountAutoCode (account, depth);
-    xaccGroupAutoCode (account->children, depth);
-  }
-} 
-
 /********************************************************************\
 \********************************************************************/
 
@@ -862,7 +743,9 @@ xaccGroupConcatGroup (AccountGroup *togrp, AccountGroup *fromgrp)
   if (!fromgrp) return;
 
   /* The act of inserting the account into togrp also causes it to
-   * automatically be deleted from fromgrp. Be careful! */
+   * automatically be deleted from fromgrp.  This causes linked 
+   * lists to be re-written, and so a cursor traversal is not safe.
+   * Be careful! */
 
   while (TRUE)
   {
@@ -871,8 +754,7 @@ xaccGroupConcatGroup (AccountGroup *togrp, AccountGroup *fromgrp)
     GList *next;
 
     accounts = fromgrp->accounts;
-    if (!accounts)
-      return;
+    if (!accounts) return;
 
     next = accounts->next;
 
@@ -880,11 +762,56 @@ xaccGroupConcatGroup (AccountGroup *togrp, AccountGroup *fromgrp)
 
     xaccGroupInsertAccount (togrp, account);
 
-    if (!next)
-      return;
+    if (!next) return;
   }
 }
 
+void
+xaccGroupCopyGroup (AccountGroup *to, AccountGroup *from)
+{
+   int i;
+   GList *node;
+   if (!to || !from) return;
+   if (!from->accounts || !to->book) return;
+
+   ENTER (" ");
+   for (node = from->accounts; node; node=node->next)
+   {
+      Account *to_acc, *from_acc = node->data;
+
+      /* This will copy the basic data and the KVP.  It will
+       * not copy any splits/transactions. It will gemini. */
+      to_acc = xaccCloneAccount (from_acc, to->book);
+
+      xaccAccountBeginEdit (to_acc);
+      to->accounts = g_list_insert_sorted (to->accounts, to_acc,
+                                          group_sort_helper);
+
+      to_acc->parent = to;
+      to_acc->core_dirty = TRUE;
+
+      /* copy child accounts too. */
+      if (from_acc->children)
+      {
+         to_acc->children = xaccMallocAccountGroup (to->book);
+         to_acc->children->parent = to_acc;
+         xaccGroupCopyGroup (to_acc->children, from_acc->children);
+      }
+      xaccAccountCommitEdit (to_acc);
+      gnc_engine_generate_event (&to_acc->guid, GNC_EVENT_CREATE);
+
+      /* make sure that we have a symmetric, uniform number of 
+       * begin-edits, so that subsequent GroupCommitEdit's 
+       * balance out. */
+      for (i=0; i<to->editlevel; i++)
+      {
+         xaccAccountBeginEdit (to_acc);
+         xaccAccountGroupBeginEdit (to_acc->children);
+      }
+   }
+   LEAVE (" ");
+}
+
 /********************************************************************\
 \********************************************************************/
 
@@ -910,10 +837,8 @@ xaccGroupMergeAccounts (AccountGroup *grp)
                             xaccAccountGetCode(acc_b))) &&
           (0 == safe_strcmp(xaccAccountGetDescription(acc_a),
                             xaccAccountGetDescription(acc_b))) &&
-          (gnc_commodity_equiv(xaccAccountGetCurrency(acc_a),
-                               xaccAccountGetCurrency(acc_b))) &&
-          (gnc_commodity_equiv(xaccAccountGetSecurity(acc_a),
-                               xaccAccountGetSecurity(acc_b))) &&
+          (gnc_commodity_equiv(xaccAccountGetCommodity(acc_a),
+                               xaccAccountGetCommodity(acc_b))) &&
           (0 == safe_strcmp(xaccAccountGetNotes(acc_a),
                             xaccAccountGetNotes(acc_b))) &&
           (xaccAccountGetType(acc_a) == xaccAccountGetType(acc_b)))
@@ -956,7 +881,7 @@ xaccGroupMergeAccounts (AccountGroup *grp)
 
           gnc_engine_generate_event (&xaccSplitGetAccount(split)->guid,
                                      GNC_EVENT_MODIFY);
-          xaccSplitSetAccount(split, NULL);
+          split->acc = NULL;
           xaccAccountInsertSplit (acc_a, split);
         }
 
@@ -1004,20 +929,6 @@ xaccGroupGetParentAccount (AccountGroup * grp)
   return grp->parent;
 }
 
-double
-DxaccGroupGetBalance (AccountGroup * grp)
-{
-  return gnc_numeric_to_double (xaccGroupGetBalance (grp));
-}
-
-gnc_numeric
-xaccGroupGetBalance (AccountGroup * grp)
-{
-  if (!grp) return gnc_numeric_zero ();
-
-  return grp->balance;
-}
-
 /********************************************************************\
 \********************************************************************/
 
@@ -1201,6 +1112,7 @@ xaccGroupVisitUnvisitedTransactions_thunk(Transaction *trn,
 {
     gpointer test_trn;
     struct group_visit_data *grdata = (struct group_visit_data*)data;
+    gboolean result;
 
     test_trn = g_hash_table_lookup(grdata->visit_table, trn);
 
@@ -1208,10 +1120,12 @@ xaccGroupVisitUnvisitedTransactions_thunk(Transaction *trn,
     {
         g_hash_table_insert(grdata->visit_table, trn, "");
 
-        grdata->proc(trn, grdata->up_data);
+        result = grdata->proc(trn, grdata->up_data);
     }
+    else
+      result = TRUE;
 
-    return TRUE;
+    return result;
 }
 
 gboolean
@@ -1274,12 +1188,12 @@ xaccGroupForEachTransaction (AccountGroup *g,
 /********************************************************************\
 \********************************************************************/
 
-GSList *
+AccountList *
 xaccGroupMapAccounts (AccountGroup *grp,
                       gpointer (*thunk)(Account *a, gpointer data),
                       gpointer data)
 {
-  GSList *result = NULL;
+  GList *result = NULL;
   GList *node;
 
   if (!grp) return(NULL);
@@ -1291,10 +1205,12 @@ xaccGroupMapAccounts (AccountGroup *grp,
     gpointer thunk_result = thunk (account, data);
 
     if (thunk_result)
-      result = g_slist_prepend (result, thunk_result);
+    {
+      result = g_list_append (result, thunk_result);
+    }
   }
 
-  return(g_slist_reverse (result));
+  return (result);
 }
 
 gpointer
@@ -1326,3 +1242,5 @@ xaccGroupForEachAccount (AccountGroup *grp,
 
   return(NULL);
 }
+
+/* ========================= END OF FILE ======================== */
