@@ -33,31 +33,65 @@
  *
  */
 
-#ifndef __XACC_ACCOUNT_GROUP_P_H__
-#define __XACC_ACCOUNT_GROUP_P_H__
+#ifndef XACC_GROUP_P_H
+#define XACC_GROUP_P_H
 
 #include "config.h"
 
 #include "BackendP.h"
-#include "GNCId.h"
+#include "GNCIdP.h"
 #include "Transaction.h"
+#include "gnc-book.h"
 #include "gnc-numeric.h"
 
 
 /** STRUCTS *********************************************************/
-struct _account_group
+struct account_group_s
 {
   /* The flags: */
   unsigned int saved : 1;
 
-  Account *parent;                 /* back-pointer to parent */
+  Account *parent;         /* back-pointer to parent */
 
-  GList *accounts;                 /* list of account pointers */
+  AccountList *accounts;   /* list of account pointers */
 
-  Backend *backend;                /* persistant storage backend */
+  GNCBook *book;           /* The book which this group belongs to */
 
-  /* cached parameters */
-  gnc_numeric balance;
+  /* keep track of nesting level of begin/end edit calls */
+  gint32 editlevel;
 };
 
-#endif /* __XACC_ACCOUNT_GROUP_P_H__ */
+/*
+ * The xaccAccountRemoveGroup() subroutine will remove the indicated
+ *    account group from its parent account. It will NOT free the
+ *    associated memory or otherwise alter the account group: the
+ *    account group can now be reparented to a new location.
+ *    Note, however, that it will mark the old parents as having
+ *    been modified.
+ *
+ * The xaccGroupRemoveAccount() subroutine will remove the indicated
+ *    account from its parent account group. It will NOT free the
+ *    associated memory or otherwise alter the account: the account
+ *    can now be reparented to a new location.
+ *    Note, however, that it will mark the old parents as having
+ *    been modified.
+ *
+ * Both of the above routines are private routines, since they are slightly
+ * dangerous:  If the removed group/account is not immediately reparented, 
+ * it can be lost, i.e. turn into a memory leak.  If the GUI or other engine
+ * user needs to move an account or group from here to there, it should use
+ * the 'Insert' routines, such as xaccGroupInsertAccount(), to make the move.
+ * The 'Insert' routines will automatically remove the account from its
+ * previous location.
+ */
+
+void    xaccAccountRemoveGroup (Account *acc);
+void    xaccGroupRemoveAccount (AccountGroup *grp, Account *account);
+
+/*
+ * The xaccFreeAccountGroup() subroutine will  ...
+ * 
+ */
+void     xaccFreeAccountGroup (AccountGroup *account_group);
+
+#endif
