@@ -34,6 +34,8 @@
  * Copyright (c) 1998, 1999, 2000 Linas Vepstas
  */
 
+#include "config.h"
+
 #include <glib.h>
 #include <stdio.h>
 #include <string.h>
@@ -46,21 +48,21 @@
 #include "TransactionP.h"
 #include "gnc-engine-util.h"
 #include "messages.h"
+#include "gnc-commodity.h"
 
 static short module = MOD_SCRUB;
 static Account * GetOrMakeAccount (AccountGroup *root, Transaction *trans,
-                                   const char *name_root);
+                                   const char *name_root, GNCBook *book);
 
 /* ================================================================ */
 
 void
-xaccGroupScrubOrphans (AccountGroup *grp)
+xaccGroupScrubOrphans (AccountGroup *grp, GNCBook *book)
 {
   GList *list;
   GList *node;
 
-  if (!grp)
-    return;
+  if (!grp || !book) return;
 
   list = xaccGroupGetAccountList (grp);
 
@@ -68,28 +70,26 @@ xaccGroupScrubOrphans (AccountGroup *grp)
   {
     Account *account = node->data;
 
-    xaccAccountTreeScrubOrphans (account);
+    xaccAccountTreeScrubOrphans (account, book);
   }
 }
 
 void
-xaccAccountTreeScrubOrphans (Account *acc)
+xaccAccountTreeScrubOrphans (Account *acc, GNCBook *book)
 {
-  if (!acc)
-    return;
+  if (!acc || !book) return;
 
-  xaccGroupScrubOrphans (xaccAccountGetChildren(acc));
-  xaccAccountScrubOrphans (acc);
+  xaccGroupScrubOrphans (xaccAccountGetChildren(acc), book);
+  xaccAccountScrubOrphans (acc, book);
 }
 
 void
-xaccAccountScrubOrphans (Account *acc)
+xaccAccountScrubOrphans (Account *acc, GNCBook *book)
 {
   GList *node;
   const char *str;
 
-  if (!acc)
-    return;
+  if (!acc || !book) return;
 
   str = xaccAccountGetName (acc);
   str = str ? str : "(null)";
@@ -100,17 +100,18 @@ xaccAccountScrubOrphans (Account *acc)
     Split *split = node->data;
 
     xaccTransScrubOrphans (xaccSplitGetParent (split),
-                           xaccGetAccountRoot (acc));
+                           xaccAccountGetRoot (acc),
+                           book);
   }
 }
 
 void
-xaccTransScrubOrphans (Transaction *trans, AccountGroup *root)
+xaccTransScrubOrphans (Transaction *trans, AccountGroup *root,
+                       GNCBook *book)
 {
   GList *node;
 
-  if (!trans)
-    return;
+  if (!trans || !book) return;
 
   for (node = xaccTransGetSplitList (trans); node; node = node->next)
   {
@@ -124,7 +125,7 @@ xaccTransScrubOrphans (Transaction *trans, AccountGroup *root)
 
     DEBUG ("Found an orphan \n");
 
-    orph = GetOrMakeAccount (root, trans, _("Orphan"));
+    orph = GetOrMakeAccount (root, trans, _("Orphan"), book);
     if (!orph)
       continue;
 
@@ -189,6 +190,8 @@ xaccSplitScrub (Split *split)
   Transaction *trans;
   gnc_numeric value;
   gboolean trans_was_open;
+  gnc_commodity *commodity;
+  gnc_commodity *currency;
   int scu;
 
   if (!split)
@@ -203,26 +206,28 @@ xaccSplitScrub (Split *split)
   {
     value = xaccSplitGetValue (split);
 
-    if (gnc_numeric_same (xaccSplitGetShareAmount (split),
+    if (gnc_numeric_same (xaccSplitGetAmount (split),
                           xaccSplitGetValue (split),
                           value.denom, GNC_RND_ROUND))
       return;
 
-    xaccSplitSetShareAmount (split, value);
+    xaccSplitSetAmount (split, value);
 
     return;
   }
 
-  if (!gnc_commodity_equiv (xaccAccountGetCurrency (account),
-                            xaccAccountGetCommodity (account)))
+  commodity = xaccAccountGetCommodity (account);
+  currency = xaccTransGetCurrency (trans);
+
+  if (!commodity || !gnc_commodity_equiv (commodity, currency))
     return;
 
-  scu = MIN (xaccAccountGetCurrencySCU (account),
-             xaccAccountGetCommoditySCU (account));
+  scu = MIN (xaccAccountGetCommoditySCU (account),
+             gnc_commodity_get_fraction (currency));
 
   value = xaccSplitGetValue (split);
 
-  if (gnc_numeric_same (xaccSplitGetShareAmount (split),
+  if (gnc_numeric_same (xaccSplitGetAmount (split),
                         value, scu, GNC_RND_ROUND))
     return;
 
@@ -233,7 +238,7 @@ xaccSplitScrub (Split *split)
   if (!trans_was_open)
     xaccTransBeginEdit (trans);
 
-  xaccSplitSetShareAmount (split, value);
+  xaccSplitSetAmount (split, value);
 
   if (!trans_was_open)
     xaccTransCommitEdit (trans);
@@ -242,12 +247,12 @@ xaccSplitScrub (Split *split)
 /* ================================================================ */
 
 void
-xaccGroupScrubImbalance (AccountGroup *grp)
+xaccGroupScrubImbalance (AccountGroup *grp, GNCBook *book)
 {
   GList *list;
   GList *node;
 
-  if (!grp) return;
+  if (!grp || !book) return;
 
   list = xaccGroupGetAccountList (grp);
 
@@ -255,23 +260,27 @@ xaccGroupScrubImbalance (AccountGroup *grp)
   {
     Account *account = node->data;
 
-    xaccAccountTreeScrubImbalance (account);
+    xaccAccountTreeScrubImbalance (account, book);
   }
 }
 
 void
-xaccAccountTreeScrubImbalance (Account *acc)
+xaccAccountTreeScrubImbalance (Account *acc, GNCBook *book)
 {
-  xaccGroupScrubImbalance (xaccAccountGetChildren(acc));
-  xaccAccountScrubImbalance (acc);
+  g_return_if_fail (book);
+
+  xaccGroupScrubImbalance (xaccAccountGetChildren(acc), book);
+  xaccAccountScrubImbalance (acc, book);
 }
 
 void
-xaccAccountScrubImbalance (Account *acc)
+xaccAccountScrubImbalance (Account *acc, GNCBook *book)
 {
   GList *node;
   const char *str;
 
+  if (!acc || !book) return;
+
   str = xaccAccountGetName(acc);
   str = str ? str : "(null)";
   PINFO ("Looking for imbalance in account %s \n", str);
@@ -281,19 +290,18 @@ xaccAccountScrubImbalance (Account *acc)
     Split *split = node->data;
     Transaction *trans = xaccSplitGetParent(split);
 
-    xaccTransScrubImbalance (trans, xaccGetAccountRoot (acc), NULL);
+    xaccTransScrubImbalance (trans, xaccAccountGetRoot (acc), NULL, book);
   }
 }
 
 void
 xaccTransScrubImbalance (Transaction *trans, AccountGroup *root,
-                         Account *parent)
+                         Account *parent, GNCBook *book)
 {
   Split *balance_split = NULL;
   gnc_numeric imbalance;
 
-  if (!trans)
-    return;
+  if (!trans || !book) return;
 
   xaccTransScrubSplits (trans);
 
@@ -306,7 +314,7 @@ xaccTransScrubImbalance (Transaction *trans, AccountGroup *root,
       return;
 
     if (!parent)
-      account = GetOrMakeAccount (root, trans, _("Imbalance"));
+      account = GetOrMakeAccount (root, trans, _("Imbalance"), book);
     else
       account = parent;
 
@@ -327,7 +335,7 @@ xaccTransScrubImbalance (Transaction *trans, AccountGroup *root,
     /* put split into account before setting split value */
     if (!balance_split)
     {
-      balance_split = xaccMallocSplit ();
+      balance_split = xaccMallocSplit (book);
 
       xaccAccountBeginEdit (account);
       xaccAccountInsertSplit (account, balance_split);
@@ -338,9 +346,10 @@ xaccTransScrubImbalance (Transaction *trans, AccountGroup *root,
   PINFO ("unbalanced transaction");
 
   {
-    const gnc_commodity *common_currency;
+    const gnc_commodity *currency;
     const gnc_commodity *commodity;
     gboolean trans_was_open;
+    gnc_numeric new_value;
     Account *account;
 
     trans_was_open = xaccTransIsOpen (trans);
@@ -348,42 +357,28 @@ xaccTransScrubImbalance (Transaction *trans, AccountGroup *root,
     if (!trans_was_open)
       xaccTransBeginEdit (trans);
 
-    common_currency = xaccTransGetCurrency (trans);
+    currency = xaccTransGetCurrency (trans);
     account = xaccSplitGetAccount (balance_split);
 
-    commodity = xaccAccountGetCurrency (account);
-    if (gnc_commodity_equiv (common_currency, commodity))
+    new_value = xaccSplitGetValue (balance_split);
+
+    /* Note: We have to round for the commodity's fraction, NOT any
+     * already existing denominator (bug #104343), because either one
+     * of the denominators might already be reduced.  */
+    new_value = gnc_numeric_sub (new_value, imbalance,
+				 gnc_commodity_get_fraction(currency), 
+				 GNC_RND_ROUND);
+
+    xaccSplitSetValue (balance_split, new_value);
+
+    commodity = xaccAccountGetCommodity (account);
+    if (gnc_commodity_equiv (currency, commodity))
+      xaccSplitSetAmount (balance_split, new_value);
+
+    if (!parent && gnc_numeric_zero_p (new_value))
     {
-      gnc_numeric new_value = xaccSplitGetValue (balance_split);
-
-      new_value = gnc_numeric_sub (new_value, imbalance,
-                                   new_value.denom, GNC_RND_ROUND);
-
-      xaccSplitSetValue (balance_split, new_value);
-
-      if (!parent && gnc_numeric_zero_p (new_value))
-      {
-        xaccSplitDestroy (balance_split);
-        balance_split = NULL;
-      }
-    }
-
-    commodity = xaccAccountGetSecurity (account);
-    if (balance_split && gnc_commodity_equiv (common_currency, commodity))
-    {
-      gnc_numeric new_share_amount = xaccSplitGetShareAmount (balance_split);
-
-      new_share_amount = gnc_numeric_sub (new_share_amount, imbalance,
-                                          new_share_amount.denom,
-                                          GNC_RND_ROUND);
-
-      xaccSplitSetShareAmount (balance_split, new_share_amount);
-
-      if (!parent && gnc_numeric_zero_p (new_share_amount))
-      {
-        xaccSplitDestroy (balance_split);
-        balance_split = NULL;
-      }
+      xaccSplitDestroy (balance_split);
+      balance_split = NULL;
     }
 
     if (balance_split)
@@ -398,14 +393,154 @@ xaccTransScrubImbalance (Transaction *trans, AccountGroup *root,
 
 /* ================================================================ */
 
+void
+xaccTransScrubCurrency (Transaction *trans, GNCBook *book)
+{
+  gnc_commodity *currency;
+
+  if (!trans || !book) return;
+
+  currency = xaccTransGetCurrency (trans);
+  if (currency) return;
+  
+  currency = xaccTransFindOldCommonCurrency (trans, book);
+  if (currency)
+  {
+    xaccTransBeginEdit (trans);
+    xaccTransSetCurrency (trans, currency);
+    xaccTransCommitEdit (trans);
+  }
+  else
+  {
+    PWARN ("no common transaction currency found");
+  }
+  {
+    Split *sp;
+    int i;
+    for (i=0; (sp = xaccTransGetSplit (trans, i)); i++) {
+      if (!gnc_numeric_equal(xaccSplitGetAmount (sp), 
+			     xaccSplitGetValue (sp))) {
+	Account *acc = xaccSplitGetAccount (sp);
+	gnc_commodity *acc_currency = xaccAccountGetCommodity (acc);
+	if (acc_currency == currency) {
+	  /* This Split needs fixing: The transaction-currency equals
+	     the account-currency/commodity, but the amount/values are
+	     inequal i.e. they still correspond to the security
+	     (amount) and the currency (value). In the new model, the
+	     value is the amount in the account-commodity -- so it
+	     needs to be set to equal the amount (since the
+	     account-currency doesn't exist anymore). 
+
+	  Note: Nevertheless we lose some information here. Namely,
+	  the information that the 'amount' in 'account-old-security'
+	  was worth 'value' in 'account-old-currency'. Maybe it would
+	  be better to store that information in the price database? 
+	  But then, for old currency transactions there is still the
+	  'other' transaction, which is going to keep that
+	  information. So I don't bother with that here. -- cstim,
+	  2002/11/20. */
+	  
+	  /*PWARN ("## Error likely: Split '%s' Amount %s %s, value %s",
+	    xaccSplitGetMemo (sp),
+	    gnc_numeric_to_string (amount),
+	    gnc_commodity_get_mnemonic (currency),
+	    gnc_numeric_to_string (value));*/
+	  xaccTransBeginEdit (trans);
+	  xaccSplitSetValue (sp, xaccSplitGetAmount (sp));
+	  xaccTransCommitEdit (trans);
+	}
+	/*else {
+	  PWARN ("Ok: Split '%s' Amount %s %s, value %s %s",
+	  xaccSplitGetMemo (sp),
+	  gnc_numeric_to_string (amount),
+	  gnc_commodity_get_mnemonic (currency),
+	  gnc_numeric_to_string (value),
+	  gnc_commodity_get_mnemonic (acc_currency));
+	  }*/
+      }
+    }
+  }
+}
+
+/* ================================================================ */
+
+void
+xaccAccountScrubCommodity (Account *account, GNCBook *book)
+{
+  gnc_commodity *commodity;
+
+  if (!account || !book) return;
+
+  commodity = xaccAccountGetCommodity (account);
+  if (commodity) return;
+
+  commodity = DxaccAccountGetSecurity (account, book);
+  if (commodity)
+  {
+    xaccAccountSetCommodity (account, commodity);
+    return;
+  }
+
+  commodity = DxaccAccountGetCurrency (account, book);
+  if (commodity)
+  {
+    xaccAccountSetCommodity (account, commodity);
+    return;
+  }
+
+  PERR ("account with no commodity");
+}
+
+/* ================================================================ */
+
+static gboolean
+scrub_trans_currency_helper (Transaction *t, gpointer data)
+{
+  GNCBook *book = data;
+
+  xaccTransScrubCurrency (t, book);
+
+  return TRUE;
+}
+
+static gpointer
+scrub_account_commodity_helper (Account *account, gpointer data)
+{
+  GNCBook *book = data;
+
+  xaccAccountScrubCommodity (account, book);
+  xaccAccountDeleteOldData (account);
+
+  return NULL;
+}
+
+void
+xaccGroupScrubCommodities (AccountGroup *group, GNCBook *book)
+{
+  if (!group || !book) return;
+
+  xaccAccountGroupBeginEdit (group);
+
+  xaccGroupForEachTransaction (group, scrub_trans_currency_helper, book);
+
+  xaccGroupForEachAccount (group, scrub_account_commodity_helper,
+                           book, TRUE);
+
+  xaccAccountGroupCommitEdit (group);
+}
+
+/* ================================================================ */
+
 static Account *
 GetOrMakeAccount (AccountGroup *root, Transaction *trans,
-                  const char *name_root)
+                  const char *name_root, GNCBook *book)
 {
   gnc_commodity * currency;
   char * accname;
   Account * acc;
 
+  g_return_val_if_fail (root, NULL);
+
   /* build the account name */
   currency = xaccTransGetCurrency (trans);
   if (!currency)
@@ -415,7 +550,7 @@ GetOrMakeAccount (AccountGroup *root, Transaction *trans,
   }
 
   accname = g_strconcat (name_root, "-",
-                         gnc_commodity_get_mnemonic(currency), NULL);
+                         gnc_commodity_get_mnemonic (currency), NULL);
 
   /* see if we've got one of these going already ... */
   acc = xaccGetAccountFromName (root, accname);
@@ -423,10 +558,10 @@ GetOrMakeAccount (AccountGroup *root, Transaction *trans,
   if (acc == NULL)
   {
     /* guess not. We'll have to build one */
-    acc = xaccMallocAccount ();
+    acc = xaccMallocAccount (book);
     xaccAccountBeginEdit (acc);
     xaccAccountSetName (acc, accname);
-    xaccAccountSetCurrency (acc, currency);
+    xaccAccountSetCommodity (acc, currency);
     xaccAccountSetType (acc, BANK);
 
     /* hang the account off the root */
