@@ -1,7 +1,4 @@
 /********************************************************************\
- * TransLog.h -- the transaction logger                             *
- * Copyright (C) 1998 Linas Vepstas                                 *
- *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
  * published by the Free Software Foundation; either version 2 of   *
@@ -21,8 +18,20 @@
  *                                                                  *
 \********************************************************************/
 
-#ifndef __XACC_TRANS_LOG_H__
-#define __XACC_TRANS_LOG_H__
+/** @addtogroup Engine
+    @{ */
+/** @file TransLog.h
+    @brief API for the transaction logger
+*
+ * The logfiles are useful for tracing, journalling, error recovery.
+ * Note that the current support for journalling is at best
+ * embryonic, at worst, is dangerous by setting the wrong expectations.
+ *
+    @author Copyright (C) 1998 Linas Vepstas
+*/
+
+#ifndef XACC_TRANS_LOG_H
+#define XACC_TRANS_LOG_H
 
 #include "config.h"
 
@@ -31,16 +40,33 @@
 
 void    xaccOpenLog (void);
 void    xaccCloseLog (void);
+/**
+ \param char The engine currently uses the log mechanism with flag char set as
+ * follows:
+ * 'B' for 'begin edit' (followed by the transaction as it looks
+ *     before any changes, i.e. the 'old value')
+ * 'D' for delete (i.e. delete the previous B; echoes the data in the
+ *     'old B')
+ * 'C' for commit (i.e. accept a previous B; data that follows is the
+ *     'new value')
+ * 'R' for rollback (i.e. revert to previous B; data that follows should
+ *     be identical to old B)
+ */
 void    xaccTransWriteLog (Transaction *, char);
+
+/** document me */
 void    xaccLogEnable (void);
+
+/** document me */
 void    xaccLogDisable (void);
 
-/* The xaccLogSetBaseName() method sets the base filepath and the
+/** The xaccLogSetBaseName() method sets the base filepath and the
  *    root part of the journal file name.  If the journal file is
  *    already open, it will close it and reopen it with the new
  *    base name.
  */
 void    xaccLogSetBaseName (const char *);
 
-#endif /* __XACC_TRANS_LOG_H__ */
+#endif /* XACC_TRANS_LOG_H */
+/** @} */
 
