@@ -42,17 +42,17 @@
  *
  */
 
-#ifndef __XACC_TRANSACTION_P_H__
-#define __XACC_TRANSACTION_P_H__
+#ifndef XACC_TRANSACTION_P_H
+#define XACC_TRANSACTION_P_H
 
 #include <time.h>
 #include <glib.h>
 
 #include "config.h"
-#include "kvp_frame.h"
+#include "gnc-engine.h"   /* for typedefs */
 #include "gnc-numeric.h"
-#include "Transaction.h"   /* for typedefs */
-#include "GNCId.h"
+#include "GNCIdP.h"
+#include "kvp_frame.h"
 
 
 /** STRUCTS *********************************************************/
@@ -74,13 +74,16 @@
  */
 
 
-struct _split 
+struct split_s
 {
   GUID guid;  /* globally unique id */
 
-  GUID acc_guid;             /* the guid of the associated account */
+  GNCBook *book;             /* The enitity table where this split is stored. */
+
   Account *acc;              /* back-pointer to debited/credited account  */
 
+  GNCLot *lot;               /* back-pointer to debited/credited lot */
+
   Transaction *parent;       /* parent of split                           */
 
   /* The memo field is an arbitrary user-assiged value. 
@@ -105,14 +108,11 @@ struct _split
   char    reconciled;        /* The reconciled field                      */
   Timespec date_reconciled;  /* date split was reconciled                 */
 
-  /* New semantics: 'value' is the amount of the transaction balancing
-   * commodity (i.e. currency) involved, 'damount' is the amount of
-   * the account's commodity (formerly known as 'security') involved.
-   *
-   * Old semantics: value is the amount of the account's currency 
-   * involved, damount is the amount of the account's security.  */
-  gnc_numeric  value;         
-  gnc_numeric  damount;  
+  /* 'value' is the amount of the transaction balancing commodity
+   * (i.e. currency) involved, 'amount' is the amount of the account's
+   * commodity (formerly known as 'security') involved. */
+  gnc_numeric  value;
+  gnc_numeric  amount;
 
   /* -------------------------------------------------------------- */
   /* Below follow some 'temporary' fields */
@@ -125,23 +125,21 @@ struct _split
   gnc_numeric  cleared_balance;
   gnc_numeric  reconciled_balance;
 
-  gnc_numeric  share_balance;
-  gnc_numeric  share_cleared_balance;
-  gnc_numeric  share_reconciled_balance;
-
   /* -------------------------------------------------------------- */
   /* Backend private expansion data */
   guint32  idata;     /* used by the sql backend for kvp management */
 };
 
 
-struct _transaction 
+struct transaction_s
 {
   /* guid is a globally unique identifier which can be used to
    * reference the transaction.
    */
   GUID guid;
 
+  GNCBook *book;         /* The entity_table where the transaction is stored */
+
   Timespec date_entered;     /* date register entry was made              */
   Timespec date_posted;      /* date transaction was posted at bank       */
 
@@ -173,7 +171,7 @@ struct _transaction
    *
    * Alternate, better(?) name: "valuation currency": it is the
    * currency in which all of the splits can be valued.  */
-  const gnc_commodity *common_currency;
+  gnc_commodity *common_currency;
 
   /* version number, used for tracking multiuser updates */
   gint32 version;
@@ -217,10 +215,21 @@ void xaccSplitSetGUID (Split *split, const GUID *guid);
  * with the split.  It does not verify that the split isn't
  * referenced in some account.  If the split is referenced by an 
  * account, then calling this method will leave the system in an 
- * inconsistent state.
+ * inconsistent state.  This *will* lead to crashes and hangs.
  */
 void  xaccFreeSplit (Split *split);    /* frees memory */
 
+/* This routine makes a 'duplicate' of the indicated transaction.
+ * This routine cannot be exposed publically since the duplicate
+ * is wrong in many ways: it is not issued a unique guid, and thus
+ * not a properly registered Entity.  The splits are copied, but
+ * these are also funny: they aren't inserted into the accounts 
+ * they claim to be in.  The splits also have bogus GUID's.
+ * Another 'feature': the splits point at the old transaction
+ * as the parent, not the new transaction.  
+ */
+Transaction * xaccDupeTransaction (Transaction *t);
+
 /* compute the value of a list of splits in the given currency,
  * excluding the skip_me split. */
 gnc_numeric xaccSplitsComputeValue (GList *splits, Split * skip_me,
@@ -232,6 +241,18 @@ gnc_numeric xaccSplitsComputeValue (GList *splits, Split * skip_me,
  *    want anyone except the backend to mess with them.
  */
 void xaccTransSetVersion (Transaction*, gint32);
-gint32 xaccTransGetVersion (Transaction*);
+gint32 xaccTransGetVersion (const Transaction*);
 
-#endif /* __XACC_TRANSACTION_P_H__ */
+/* The xaccTransFindCommonCurrency () method returns a gnc_commodity
+ *    indicating a currency denomination that all of the splits in this
+ *    transaction have in common, using the old currency/security fields
+ *    of the split accounts. */
+gnc_commodity * xaccTransFindOldCommonCurrency (Transaction *trans,
+                                                GNCBook *book);
+
+/* Code to register Split and Transaction types with the engine */
+gboolean xaccSplitRegister (void);
+gboolean xaccTransRegister (void);
+
+
+#endif /* XACC_TRANSACTION_P_H */
