@@ -46,6 +46,13 @@
 #include "date.h"
 #include "gnc-engine-util.h"
 
+#ifndef HAVE_STRPTIME
+#include "strptime.h"
+#endif
+#ifndef HAVE_LOCALTIME_R
+#include "localtime_r.h"
+#endif
+
 #define NANOS_PER_SECOND 1000000000
 
 #ifdef HAVE_LANGINFO_D_FMT
@@ -55,7 +62,8 @@
 #endif
 
 /* This is now user configured through the gnome options system() */
-static DateFormat dateFormat = DATE_FORMAT_US;
+static DateFormat dateFormat = DATE_FORMAT_LOCALE;
+static DateFormat prevDateFormat = DATE_FORMAT_LOCALE;
 
 /* This static indicates the debugging module that this .o belongs to. */
 static short module = MOD_ENGINE;
@@ -153,15 +161,99 @@ timespecCanonicalDayTime(Timespec t)
   time_t t_secs = t.tv_sec + (t.tv_nsec / NANOS_PER_SECOND);
   result = localtime(&t_secs);
   tm = *result;
-  tm.tm_sec = 0;
-  tm.tm_min = 0;
-  tm.tm_hour = 12;
-  tm.tm_isdst = -1;
+  gnc_tm_set_day_middle(&tm);
   retval.tv_sec = mktime(&tm);
   retval.tv_nsec = 0;
   return retval;
 }
-    
+
+int gnc_date_my_last_mday (int month, int year)
+{
+  gboolean is_leap;
+  static int days_in_month[2][12] =
+    {/* non leap */ {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
+     /*   leap   */ {31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}};
+
+  /* Is this a leap year? */
+  if (year / 2000 == 0)
+    is_leap = TRUE;
+  else if (year / 400 == 0)
+      is_leap = FALSE;
+  else
+    is_leap = (year / 4 == 0);
+
+  return days_in_month[is_leap][month-1];
+}
+
+/**
+ * date_get_last_mday
+ * Retrieve the last nomerical day for the month specified in the
+ * tm_year and tm_mon fields.
+ * Args:  tm: the time value in question
+ * returns: T/F
+ **/
+int date_get_last_mday(struct tm *tm)
+{
+  return gnc_date_my_last_mday (tm->tm_mon+1, tm->tm_year+1900);
+}
+
+/**
+ * date_is_last_mday
+ * Determines whether the tm_mday field contains the last day of the
+ * month as specified in the tm_year and tm_mon fields.
+ * Args:  tm: the time value in question
+ * returns: T/F
+ **/
+gboolean date_is_last_mday(struct tm *tm)
+{
+  return(tm->tm_mday == date_get_last_mday(tm));
+}
+
+/**
+ * date_add_months
+ * Add a number of months to a time value, and normalize.  Optionally
+ * also track the last day of hte month, i.e. 1/31 -> 2/28 -> 3/30.
+ * Args:  tm: base time value
+ *        months: The number of months to add to this time
+ *        track_last_day: Coerce the date value if necessary.
+ * returns: nothing
+ **/
+void date_add_months (struct tm *tm, int months, gboolean track_last_day)
+{
+  gboolean was_last_day;
+  int new_last_mday;
+
+  /* Have to do this now */
+  was_last_day = date_is_last_mday(tm);
+
+  /* Add in the months and normalize */
+  tm->tm_mon += months;
+  while (tm->tm_mon > 11) {
+    tm->tm_mon -= 12;
+    tm->tm_year++;
+  }
+
+  if (!track_last_day)
+    return;
+
+  /* Track last day of the month, i.e. 1/31 -> 2/28 -> 3/30 */
+  new_last_mday = date_get_last_mday(tm);
+  if (was_last_day || (tm->tm_mday > new_last_mday))
+    tm->tm_mday = new_last_mday;
+}
+
+/**
+ * getDateFormat
+ * Args: nothing
+ * returns: DateFormat: enumeration indicating preferred format
+ *
+ * Globals: dateFormat
+ **/
+DateFormat getDateFormat (void)
+{
+  return dateFormat;
+}
+
 /**
  * setDateFormat
  * set date format to one of US, UK, CE, OR ISO
@@ -171,11 +263,11 @@ timespecCanonicalDayTime(Timespec t)
  *
  * Globals: dateFormat
  **/
-
 void setDateFormat(DateFormat df)
 {
   if(df >= DATE_FORMAT_FIRST && df <= DATE_FORMAT_LAST)
   {
+    prevDateFormat = dateFormat;
     dateFormat = df;
   }
   else
@@ -186,6 +278,53 @@ void setDateFormat(DateFormat df)
   return;
 }
 
+/**
+ * getDateFormatString
+ * get the date format string for the current format
+ * returns: string
+ *
+ * Globals: dateFormat
+ **/
+const gchar *getDateFormatString(DateFormat df)
+{
+  switch(df) {
+   case DATE_FORMAT_US:
+    return "%m/%d/%y";
+   case DATE_FORMAT_UK:
+    return "%d/%m/%y";
+   case DATE_FORMAT_CE:
+    return "%d.%m.%y";
+   case DATE_FORMAT_ISO:
+    return "%y-%m-%d";
+   case DATE_FORMAT_LOCALE:
+   default:
+    return GNC_D_FMT;
+  };
+}
+
+/**
+ * getDateTextFormatString
+ * get the date format string for the current format
+ * returns: string
+ *
+ * Globals: dateFormat
+ **/
+const gchar *getDateTextFormatString(DateFormat df)
+{
+  switch(df) {
+   case DATE_FORMAT_US:
+    return "%b %d, %y";
+   case DATE_FORMAT_UK:
+   case DATE_FORMAT_CE:
+    return "%d %b, %y";
+   case DATE_FORMAT_ISO:
+    return "%y-%b-%d";
+   case DATE_FORMAT_LOCALE:
+   default:
+    return GNC_D_FMT;
+  };
+}
+
 /**
  * printDate
  *    Convert a date as day / month / year integers into a localized string
@@ -233,11 +372,8 @@ printDate (char * buff, int day, int month, int year)
         tm_str.tm_mon = month - 1;    /* tm_mon = 0 through 11 */
         tm_str.tm_year = year - 1900; /* this is what the standard 
                                        * says, it's not a Y2K thing */
-        tm_str.tm_hour = 0;
-        tm_str.tm_min = 0;
-        tm_str.tm_sec = 0;
-        tm_str.tm_isdst = -1;
 
+	gnc_tm_set_day_start (&tm_str);
         strftime (buff, MAX_DATE_LENGTH, GNC_D_FMT, &tm_str);
       }
       break;
@@ -263,6 +399,15 @@ printDateSecs (char * buff, time_t t)
                    theTime->tm_year + 1900);
 }
 
+void
+printGDate( char *buf, GDate *gd )
+{
+  printDate( buf,
+             g_date_day(gd),
+             g_date_month(gd),
+             g_date_year(gd) );
+}
+
 char * 
 xaccPrintDateSecs (time_t t)
 {
@@ -289,6 +434,12 @@ gnc_print_date (Timespec ts)
  *    Convert a string into  day / month / year integers according to
  *    the current dateFormat value.
  *
+ *    This function will always parse a single number as the day of
+ *    the month, regardless of the ordering of the dateFormat value.
+ *    Two numbers will always be parsed as the day and the month, in
+ *    the same order that they appear in the dateFormat value.  Three
+ *    numbers are parsed exactly as specified in the dateFormat field.
+ *
  * Args:   buff - pointer to date string
  *         day -  will store day of the month as 1 ... 31
  *         month - will store month of the year as 1 ... 12
@@ -298,15 +449,16 @@ gnc_print_date (Timespec ts)
  *
  * Globals: global dateFormat value
  */
-void
-scanDate (const char *buff, int *day, int *month, int *year)
+static gboolean
+scanDateInternal (const char *buff, int *day, int *month, int *year,
+		  DateFormat which_format)
 {
    char *dupe, *tmp, *first_field, *second_field, *third_field;
    int iday, imonth, iyear;
    struct tm *now;
    time_t secs;
 
-   if (!buff) return;
+   if (!buff) return(FALSE);
 
    dupe = g_strdup (buff);
 
@@ -317,11 +469,13 @@ scanDate (const char *buff, int *day, int *month, int *year)
 
    /* use strtok to find delimiters */
    if (tmp) {
-      first_field = strtok (tmp, ".,-+/\\()");
+     static char *delims = ".,-+/\\() ";
+
+      first_field = strtok (tmp, delims);
       if (first_field) {
-         second_field = strtok (NULL, ".,-+/\\()");
+         second_field = strtok (NULL, delims);
          if (second_field) {
-            third_field = strtok (NULL, ".,-+/\\()");
+            third_field = strtok (NULL, delims);
          }
       }
    }
@@ -334,41 +488,108 @@ scanDate (const char *buff, int *day, int *month, int *year)
    iyear = now->tm_year+1900;
 
    /* get numeric values */
-   switch (dateFormat)
+   switch (which_format)
    {
      case DATE_FORMAT_LOCALE:
        if (buff[0] != '\0')
        {
          struct tm thetime;
 
+	 /* Parse time string. */
+	 memset(&thetime, -1, sizeof(struct tm));
          strptime (buff, GNC_D_FMT, &thetime);
 
-         iday = thetime.tm_mday;
-         imonth = thetime.tm_mon + 1;
-         iyear = thetime.tm_year + 1900;
+	 if (third_field) {
+	   /* Easy.  All three values were parsed. */
+	   iyear = thetime.tm_year + 1900;
+	   iday = thetime.tm_mday;
+	   imonth = thetime.tm_mon + 1;
+	 } else if (second_field) {
+	   /* Hard. Two values parsed.  Figure out the ordering. */
+	   if (thetime.tm_year == -1) {
+	     /* %m-%d or %d-%m. Don't care. Already parsed correctly. */
+	     iday = thetime.tm_mday;
+	     imonth = thetime.tm_mon + 1;
+	   } else if (thetime.tm_mon != -1) {
+	     /* Must be %Y-%m-%d. Reparse as %m-%d.*/
+	     imonth = atoi(first_field);
+	     iday = atoi(second_field);
+	   } else {
+	     /* Must be %Y-%d-%m. Reparse as %d-%m. */
+	     iday = atoi(first_field);
+	     imonth = atoi(second_field);
+	   }
+	 } else if (first_field) {
+	   iday = atoi(first_field);
+	 }
        }
        break;
      case DATE_FORMAT_UK:
      case DATE_FORMAT_CE:
-       if (first_field) iday = atoi (first_field);
-       if (second_field) imonth = atoi (second_field);
-       if (third_field) iyear = atoi (third_field);
+       if (third_field) {
+	 iday = atoi(first_field);
+         imonth = atoi(second_field);
+         iyear = atoi(third_field);
+       } else if (second_field) {
+         iday = atoi(first_field);
+         imonth = atoi(second_field);
+       } else if (first_field) {
+         iday = atoi(first_field);
+       }
        break;
      case DATE_FORMAT_ISO:
-       if (first_field) iyear = atoi (first_field);
-       if (second_field) imonth = atoi (second_field);
-       if (third_field) iday = atoi (third_field);
+       if (third_field) {
+	 iyear = atoi(first_field);
+         imonth = atoi(second_field);
+         iday = atoi(third_field);
+       } else if (second_field) {
+         imonth = atoi(first_field);
+         iday = atoi(second_field);
+       } else if (first_field) {
+         iday = atoi(first_field);
+       }
        break;
-     case DATE_FORMAT_US:
-     default:
-       if (first_field) imonth = atoi (first_field);
-       if (second_field) iday = atoi (second_field);
-       if (third_field) iyear = atoi (third_field);
+    case DATE_FORMAT_US:
+    default:
+       if (third_field) {
+         imonth = atoi(first_field);
+         iday = atoi(second_field);
+         iyear = atoi(third_field);
+       } else if (second_field) {
+         imonth = atoi(first_field);
+         iday = atoi(second_field);
+       } else if (first_field) {
+         iday = atoi(first_field);
+       }
        break;
    }
 
    g_free (dupe);
 
+   if (imonth > 12 || iday > 31) {
+     /* 
+      * Ack! Thppfft!  Someone just fed this routine a string in the
+      * wrong date format.  This is known to happen if a register
+      * window is open when changing the date format.  Try the
+      * previous date format.  If that doesn't work, bail and give the
+      * caller what they asked for (garbage) parsed in the new format.
+      *
+      * Note: This test cannot detect any format change that only
+      * swaps month and day field, if the day is 12 or less.  This is
+      * deemed acceptable given the obscurity of this bug.
+      */
+     if ((which_format != prevDateFormat)
+	 && (scanDateInternal(buff, day, month, year, prevDateFormat)))
+       return(TRUE);
+
+     if (imonth > 12 && iday <= 12) {
+       int tmp = imonth;
+       imonth = iday;
+       iday = tmp;
+     } else
+       return FALSE;
+   }
+
    /* if the year entered is smaller than 100, assume we mean the current
       century (and are not revising some roman emperor's books) */
    if (iyear < 100)
@@ -377,6 +598,13 @@ scanDate (const char *buff, int *day, int *month, int *year)
    if (year) *year=iyear;
    if (month) *month=imonth;
    if (day) *day=iday;
+   return(TRUE);
+}
+
+void
+scanDate (const char *buff, int *day, int *month, int *year)
+{
+  scanDateInternal(buff, day, month, year, dateFormat);
 }
 
 /**
@@ -469,42 +697,59 @@ gnc_iso8601_to_timespec(const char *str, int do_localtime)
   }
   stm.tm_isdst = -1;
 
-  /* timezone format can be +hh or +hhmm or +hh.mm (or -) */
+  /* timezone format can be +hh or +hhmm or +hh.mm (or -) (or not present) */
   str += strcspn (str, "+-");
-  buf[0] = str[0];
-  buf[1] = str[1];
-  buf[2] = str[2];
-  buf[3] = 0;
-  stm.tm_hour -= atoi(buf);
-
-  str +=3;
-  if ('.' == *str) str++;
-  if (isdigit (*str) && isdigit (*(str+1)))
+  if (str)
   {
-     int cyn;
-     /* copy sign from hour part */
-     if ('+' == buf[0]) { cyn = -1; } else { cyn = +1; } 
-     buf[0] = str[0];
-     buf[1] = str[1];
-     buf[2] = str[2];
-     buf[3] = 0;
-     stm.tm_min += cyn * atoi(buf);
+    buf[0] = str[0];
+    buf[1] = str[1];
+    buf[2] = str[2];
+    buf[3] = 0;
+    stm.tm_hour -= atoi(buf);
+
+    str +=3;
+    if ('.' == *str) str++;
+    if (isdigit (*str) && isdigit (*(str+1)))
+    {
+      int cyn;
+      /* copy sign from hour part */
+      if ('+' == buf[0]) { cyn = -1; } else { cyn = +1; } 
+      buf[0] = str[0];
+      buf[1] = str[1];
+      buf[2] = str[2];
+      buf[3] = 0;
+      stm.tm_min += cyn * atoi(buf);
+    }
   }
 
   /* adjust for the local timezone */
   if (do_localtime)
   {
+    struct tm tmp_tm;
     struct tm *tm;
+    long int tz;
     int tz_hour;
-    time_t secs = mktime (&stm);
+    time_t secs;
 
-    /* The call to localtime is 'bogus', but it forces 'timezone' to be set.
-     * Note that we must use the accurate date, since the value of 'gnc_timezone'
-     * includes daylight savings corrections for that date. */
-    tm = localtime (&secs);   
-    tz_hour = gnc_timezone(tm)/3600;
+    /* Use a temporary tm struct so the mktime call below
+     * doesn't mess up stm. */
+    tmp_tm = stm;
+    tmp_tm.tm_isdst = -1;
+
+    secs = mktime (&tmp_tm);
+
+    /* The call to localtime is 'bogus', but it forces 'timezone' to
+     * be set. Note that we must use the accurate date, since the
+     * value of 'gnc_timezone' includes daylight savings corrections
+     * for that date. */
+    tm = localtime (&secs);
+
+    tz = gnc_timezone (tm);
+
+    tz_hour = tz / 3600;
     stm.tm_hour -= tz_hour;
-    stm.tm_min -= (gnc_timezone(tm) - 3600*tz_hour)/60;
+    stm.tm_min -= (tz - (3600 * tz_hour)) / 60;
+    stm.tm_isdst = tmp_tm.tm_isdst;
   }
 
   /* compute number of seconds */
@@ -569,6 +814,26 @@ gnc_timespec_to_iso8601_buff (Timespec ts, char * buff)
   return buff;
 }
 
+int
+gnc_timespec_last_mday (Timespec t)
+{
+  struct tm *result;
+  time_t t_secs = t.tv_sec + (t.tv_nsec / NANOS_PER_SECOND);
+  result = localtime(&t_secs);
+  return date_get_last_mday (result);
+}
+
+void
+gnc_timespec2dmy (Timespec t, int *day, int *month, int *year)
+{
+  struct tm *result;
+  time_t t_secs = t.tv_sec + (t.tv_nsec / NANOS_PER_SECOND);
+  result = localtime(&t_secs);
+
+  if (day) *day = result->tm_mday;
+  if (month) *month = result->tm_mon+1;
+  if (year) *year = result->tm_year+1900;
+}
 
 /********************************************************************\
 \********************************************************************/
@@ -584,10 +849,7 @@ xaccDMYToSec (int day, int month, int year)
   stm.tm_year = year - 1900;
   stm.tm_mon = month - 1;
   stm.tm_mday = day;
-  stm.tm_hour = 0;
-  stm.tm_min = 0;
-  stm.tm_sec = 0;
-  stm.tm_isdst = -1;
+  gnc_tm_set_day_start(&stm);
 
   /* compute number of seconds */
   secs = mktime (&stm);
@@ -632,19 +894,9 @@ gnc_dmy2timespec_internal (int day, int month, int year, gboolean start_of_day)
   date.tm_mday = day;
 
   if (start_of_day)
-  {
-    date.tm_hour = 0;
-    date.tm_min = 0;
-    date.tm_sec = 0;
-  }
+    gnc_tm_set_day_start(&date);
   else
-  {
-    date.tm_hour = 23;
-    date.tm_min = 59;
-    date.tm_sec = 59;
-  }
-
-  date.tm_isdst = -1;
+    gnc_tm_set_day_end(&date);
 
   /* compute number of seconds */
   secs = mktime (&date);
@@ -689,5 +941,115 @@ gnc_timezone (struct tm *tm)
 #endif
 }
 
+
+void
+timespecFromTime_t( Timespec *ts, time_t t )
+{
+    ts->tv_sec = t;
+    ts->tv_nsec = 0;
+}
+
+time_t 
+timespecToTime_t (Timespec ts)
+{
+    return ts.tv_sec;
+}
+
+void
+gnc_tm_get_day_start (struct tm *tm, time_t time_val)
+{
+  /* Get the equivalent time structure */
+  tm = localtime_r(&time_val, tm);
+  gnc_tm_set_day_start(tm);
+}
+
+void
+gnc_tm_get_day_end (struct tm *tm, time_t time_val)
+{
+  /* Get the equivalent time structure */
+  tm = localtime_r(&time_val, tm);
+  gnc_tm_set_day_end(tm);
+}
+
+time_t
+gnc_timet_get_day_start (time_t time_val)
+{
+  struct tm tm;
+
+  gnc_tm_get_day_start(&tm, time_val);
+  return mktime(&tm);
+}
+
+time_t
+gnc_timet_get_day_end (time_t time_val)
+{
+  struct tm tm;
+
+  gnc_tm_get_day_end(&tm, time_val);
+  return mktime(&tm);
+}
+
+/* The xaccDateUtilGetStamp() routine will take the given time in
+ * seconds and return a buffer containing a textual for the date. */
+char *
+xaccDateUtilGetStamp (time_t thyme)
+{
+   struct tm *stm;
+
+   stm = localtime (&thyme);
+
+   return g_strdup_printf("%04d%02d%02d%02d%02d%02d",
+      (stm->tm_year + 1900),
+      (stm->tm_mon +1),
+      stm->tm_mday,
+      stm->tm_hour,
+      stm->tm_min,
+      stm->tm_sec
+   );
+}
+
+/* ======================================================== */
+
+void
+gnc_tm_get_today_start (struct tm *tm)
+{
+  gnc_tm_get_day_start(tm, time(NULL));
+}
+
+void
+gnc_tm_get_today_end (struct tm *tm)
+{
+  gnc_tm_get_day_end(tm, time(NULL));
+}
+
+time_t
+gnc_timet_get_today_start (void)
+{
+  struct tm tm;
+
+  gnc_tm_get_day_start(&tm, time(NULL));
+  return mktime(&tm);
+}
+
+time_t
+gnc_timet_get_today_end (void)
+{
+  struct tm tm;
+
+  gnc_tm_get_day_end(&tm, time(NULL));
+  return mktime(&tm);
+}
+
+/* The xaccDateUtilGetStampNow() routine returns the current time in
+ * seconds in textual format. */
+char *
+xaccDateUtilGetStampNow (void)
+{
+   time_t now;
+   time (&now);
+   return xaccDateUtilGetStamp (now);
+}
+
+
 /********************** END OF FILE *********************************\
 \********************************************************************/
