@@ -55,6 +55,8 @@ struct gnc_book_struct
   /* Pointers to top-level data structures. */
   AccountGroup *topgroup;
   GNCPriceDB *pricedb;
+  gpointer shared_quickfill;
+  void (*shared_quickfill_destroy) (GNCBook *, gpointer);
 
   GList *sched_xactions;
   AccountGroup *template_group;
