@@ -1448,7 +1448,7 @@
     const char *fullname = "Israeli New Shekel";
     gnc_commodity *c = gnc_commodity_new(fullname,
                                          "ISO4217",
-                                         "ILS",
+                                         "NIS",
                                          "376",
                                          100);
 
