@@ -20,8 +20,8 @@
  * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
 \********************************************************************/
 
-#ifndef __DIALOG_PRINT_CHECK_H_
-#define __DIALOG_PRINT_CHECK_H_
+#ifndef DIALOG_PRINT_CHECK_H
+#define DIALOG_PRINT_CHECK_H
 
 #include "config.h"
 
@@ -29,14 +29,18 @@
 #include <guile/gh.h>
 
 #include "print-session.h"
-#include "glade-gnc-dialogs.h"
 
 typedef struct {
   GtkWidget * dialog;
 
+  RegWindow     *reg_data;
+  const char    *payee;
+  gnc_numeric    amount;
+  time_t         date;
+  const char    *memo;
+
   GtkWidget * format_picker;
   GtkWidget * position_picker;
-  GtkWidget * dformat_picker;
   GtkWidget * payee_x,  * payee_y;
   GtkWidget * date_x,   * date_y;
   GtkWidget * words_x,  * words_y;
@@ -47,20 +51,18 @@ typedef struct {
   GtkWidget * format_entry;
 
   GtkWidget * units_picker;
-  
-  SCM callback;
+
+  GtkWidget * date_format;
+
+  gchar *format_string;
 
 } PrintCheckDialog;
 
-PrintCheckDialog * gnc_ui_print_check_dialog_create(SCM callback);
+void gnc_ui_print_check_dialog_create(RegWindow     *reg_data,
+				      const char    *payee,
+				      gnc_numeric    amount,
+				      time_t         date,
+				      const char    *memo);
 void gnc_ui_print_check_dialog_destroy(PrintCheckDialog * pcd);
-void gnc_ui_print_check_dialog_ok_cb(GtkButton * button,
-                                     gpointer  user_data);
-
-void gnc_ui_print_check_dialog_cancel_cb(GtkButton * button,
-                                         gpointer user_data);
-
-void gnc_ui_print_check_dialog_help_cb(GtkButton * button,
-                                       gpointer user_data);
 
 #endif
