@@ -24,22 +24,24 @@
 #include "config.h"
 
 #include <gnome.h>
+#include <libgnomeui/gnome-window-icon.h>
 
-#include "FileDialog.h"
 #include "Group.h"
-#include "account-tree.h"
+#include "Transaction.h"
 #include "dialog-utils.h"
 #include "druid-utils.h"
-#include "glade-gnc-dialogs.h"
-#include "glade-support.h"
+#include "global-options.h"
+#include "gnc-account-tree.h"
 #include "gnc-amount-edit.h"
 #include "gnc-component-manager.h"
 #include "gnc-currency-edit.h"
-#include "gnc-dateedit.h"
+#include "gnc-date-edit.h"
+#include "gnc-engine-util.h"
 #include "gnc-exp-parser.h"
+#include "gnc-gui-query.h"
 #include "gnc-ui.h"
+#include "gnc-ui-util.h"
 #include "messages.h"
-#include "query-user.h"
 
 
 #define DRUID_STOCK_SPLIT_CM_CLASS "druid-stock-split"
@@ -95,12 +97,12 @@ fill_account_list (StockSplitInfo *info, Account *account)
 
   gtk_clist_clear (clist);
 
-  accounts = xaccGroupGetSubAccounts (gncGetCurrentGroup ());
+  accounts = xaccGroupGetSubAccounts (gnc_get_current_group ());
   for (node = accounts; node; node = node->next)
   {
     Account *account = node->data;
     GNCPrintAmountInfo print_info;
-    const gnc_commodity *security;
+    const gnc_commodity *commodity;
     GNCAccountType account_type;
     gnc_numeric balance;
     char *strings[4];
@@ -111,20 +113,20 @@ fill_account_list (StockSplitInfo *info, Account *account)
         account_type != MUTUAL)
       continue;
 
-    balance = xaccAccountGetShareBalance (account);
+    balance = xaccAccountGetBalance (account);
     if (gnc_numeric_zero_p (balance))
       continue;
 
     if (xaccAccountGetPlaceholder (account))
 	continue;
 
-    security = xaccAccountGetSecurity (account);
+    commodity = xaccAccountGetCommodity (account);
 
-    print_info = gnc_account_quantity_print_info (account, FALSE);
+    print_info = gnc_account_print_info (account, FALSE);
 
     strings[0] = xaccAccountGetFullName (account,
                                          gnc_get_account_separator ());
-    strings[1] = (char *) gnc_commodity_get_mnemonic (security);
+    strings[1] = (char *) gnc_commodity_get_mnemonic (commodity);
     strings[2] = (char *) xaccPrintAmount (balance, print_info);
     strings[3] = NULL;
 
@@ -177,11 +179,11 @@ refresh_details_page (StockSplitInfo *info)
   GNCPrintAmountInfo print_info;
   Account *account;
 
-  account = xaccAccountLookup (&info->account);
+  account = xaccAccountLookup (&info->account, gnc_get_current_book ());
 
   g_return_if_fail (account != NULL);
 
-  print_info = gnc_account_quantity_print_info (account, FALSE);
+  print_info = gnc_account_print_info (account, FALSE);
 
   gnc_amount_edit_set_print_info (GNC_AMOUNT_EDIT (info->distribution_edit),
                                   print_info);
@@ -190,7 +192,7 @@ refresh_details_page (StockSplitInfo *info)
 
   gnc_currency_edit_set_currency
     (GNC_CURRENCY_EDIT (info->price_currency_edit),
-     xaccAccountGetCurrency (account));
+     xaccAccountGetCommodity (account));
 }
 
 static gboolean
@@ -201,7 +203,7 @@ account_next (GnomeDruidPage *druidpage,
   StockSplitInfo *info = user_data;
   Account *account;
 
-  account = xaccAccountLookup (&info->account);
+  account = xaccAccountLookup (&info->account, gnc_get_current_book ());
 
   g_return_val_if_fail (account != NULL, TRUE);
 
@@ -214,7 +216,6 @@ static void
 gnc_parse_error_dialog (StockSplitInfo *info, const char *error_string)
 {
   const char * parse_error_string;
-  char * error_phrase;
 
   parse_error_string = gnc_exp_parser_error_string ();
   if (parse_error_string == NULL)
@@ -223,13 +224,20 @@ gnc_parse_error_dialog (StockSplitInfo *info, const char *error_string)
   if (error_string == NULL)
     error_string = "";
 
-  error_phrase = g_strdup_printf ("%s.\n\n%s: %s.",
-                                  error_string, _("Error"),
-                                  parse_error_string);
+  gnc_error_dialog_parented (GTK_WINDOW (info->window),
+			     "%s.\n\n%s: %s.",
+			     error_string, _("Error"),
+			     parse_error_string);
+}
 
-  gnc_error_dialog_parented (GTK_WINDOW (info->window), error_phrase);
+static void
+details_prepare (GnomeDruidPage *druidpage,
+		 gpointer arg1,
+		 gpointer user_data)
+{
+  StockSplitInfo *info = user_data;
 
-  g_free (error_phrase);
+  gtk_widget_grab_focus(info->distribution_edit);
 }
 
 static gboolean
@@ -291,6 +299,8 @@ cash_prepare (GnomeDruidPage *druidpage,
   gnc_account_tree_refresh (GNC_ACCOUNT_TREE (info->asset_tree));
   gnc_account_tree_expand_all (GNC_ACCOUNT_TREE (info->asset_tree));
   gtk_clist_select_row (GTK_CLIST (info->asset_tree), 0, 0);
+
+  gtk_widget_grab_focus(info->cash_edit);
 }
 
 static gboolean
@@ -361,7 +371,7 @@ stock_split_finish (GnomeDruidPage *druidpage,
   Split *split;
   time_t date;
 
-  account = xaccAccountLookup (&info->account);
+  account = xaccAccountLookup (&info->account, gnc_get_current_book ());
   g_return_if_fail (account != NULL);
 
   amount = gnc_amount_edit_get_amount
@@ -370,10 +380,12 @@ stock_split_finish (GnomeDruidPage *druidpage,
 
   gnc_suspend_gui_refresh ();
 
-  trans = xaccMallocTransaction ();
+  trans = xaccMallocTransaction (gnc_get_current_book ());
 
   xaccTransBeginEdit (trans);
 
+  xaccTransSetCurrency (trans, gnc_default_currency ());
+
   date = gnc_date_edit_get_date (GNC_DATE_EDIT (info->date_edit));
   xaccTransSetDateSecs (trans, date);
 
@@ -384,7 +396,7 @@ stock_split_finish (GnomeDruidPage *druidpage,
     xaccTransSetDescription (trans, description);
   }
 
-  split = xaccMallocSplit ();
+  split = xaccMallocSplit (gnc_get_current_book ());
 
   xaccAccountBeginEdit (account);
   account_commits = g_list_prepend (NULL, account);
@@ -393,7 +405,7 @@ stock_split_finish (GnomeDruidPage *druidpage,
 
   xaccAccountInsertSplit (account, split);
 
-  xaccSplitSetShareAmount (split, amount);
+  xaccSplitSetAmount (split, amount);
   xaccSplitMakeStockSplit (split);
   xaccSplitSetAction (split, _("Split"));
 
@@ -411,10 +423,10 @@ stock_split_finish (GnomeDruidPage *druidpage,
     ts.tv_sec = date;
     ts.tv_nsec = 0;
 
-    price = gnc_price_create ();
+    price = gnc_price_create (gnc_get_current_book ());
 
     gnc_price_begin_edit (price);
-    gnc_price_set_commodity (price, xaccAccountGetSecurity (account));
+    gnc_price_set_commodity (price, xaccAccountGetCommodity (account));
     gnc_price_set_currency (price, gnc_currency_edit_get_currency (ce));
     gnc_price_set_time (price, ts);
     gnc_price_set_source (price, "user:stock-split");
@@ -422,7 +434,7 @@ stock_split_finish (GnomeDruidPage *druidpage,
     gnc_price_set_value (price, amount);
     gnc_price_commit_edit (price);
 
-    book = gncGetCurrentBook ();
+    book = gnc_get_current_book ();
     pdb = gnc_book_get_pricedb (book);
 
     if (!gnc_pricedb_add_price (pdb, price))
@@ -443,7 +455,7 @@ stock_split_finish (GnomeDruidPage *druidpage,
     account = gnc_account_tree_get_current_account
       (GNC_ACCOUNT_TREE (info->asset_tree));
 
-    split = xaccMallocSplit ();
+    split = xaccMallocSplit (gnc_get_current_book ());
 
     xaccAccountBeginEdit (account);
     account_commits = g_list_prepend (account_commits, account);
@@ -452,7 +464,7 @@ stock_split_finish (GnomeDruidPage *druidpage,
 
     xaccTransAppendSplit (trans, split);
 
-    xaccSplitSetShareAmount (split, amount);
+    xaccSplitSetAmount (split, amount);
     xaccSplitSetValue (split, amount);
 
     xaccSplitSetMemo (split, memo);
@@ -462,7 +474,7 @@ stock_split_finish (GnomeDruidPage *druidpage,
     account = gnc_account_tree_get_current_account
       (GNC_ACCOUNT_TREE (info->income_tree));
 
-    split = xaccMallocSplit ();
+    split = xaccMallocSplit (gnc_get_current_book ());
 
     xaccAccountBeginEdit (account);
     account_commits = g_list_prepend (account_commits, account);
@@ -471,7 +483,7 @@ stock_split_finish (GnomeDruidPage *druidpage,
 
     xaccTransAppendSplit (trans, split);
 
-    xaccSplitSetShareAmount (split, gnc_numeric_neg (amount));
+    xaccSplitSetAmount (split, gnc_numeric_neg (amount));
     xaccSplitSetValue (split, gnc_numeric_neg (amount));
 
     xaccSplitSetMemo (split, memo);
@@ -496,27 +508,17 @@ druid_cancel (GnomeDruid *druid, gpointer user_data)
   gnc_close_gui_component_by_data (DRUID_STOCK_SPLIT_CM_CLASS, info);
 }
 
-static gboolean
-account_currency_filter (Account *account, gpointer user_data)
-{
-  StockSplitInfo *info = user_data;
-  Account *split_account = xaccAccountLookup (&info->account);
-
-  if (!account)
-    return FALSE;
-
-  return gnc_commodity_equiv (xaccAccountGetCurrency (split_account),
-                              xaccAccountGetCurrency (account));
-}
-
 static void
 gnc_stock_split_druid_create (StockSplitInfo *info)
 {
   GtkWidget *page;
+  GladeXML *xml;
 
-  info->window = create_Stock_Split_Druid ();
+  xml = gnc_glade_xml_new ("stocks.glade", "Stock Split Druid");
 
-  info->druid = lookup_widget (info->window, "stock_split_druid");
+  info->window = glade_xml_get_widget (xml, "Stock Split Druid");
+
+  info->druid = glade_xml_get_widget (xml, "stock_split_druid");
 
   gtk_signal_connect (GTK_OBJECT (info->window), "destroy",
                       GTK_SIGNAL_FUNC (window_destroy_cb), info);
@@ -524,14 +526,16 @@ gnc_stock_split_druid_create (StockSplitInfo *info)
   gtk_signal_connect (GTK_OBJECT (info->druid), "cancel",
                       GTK_SIGNAL_FUNC (druid_cancel), info);
 
-	gnc_druid_set_title_image (GNOME_DRUID(info->druid), "stock_split_title.png");
-	gnc_druid_set_watermark_image (GNOME_DRUID(info->druid), "stock_split_watermark.png");
+  gnc_druid_set_title_image (GNOME_DRUID(info->druid),
+                             "stock_split_title.png");
+  gnc_druid_set_watermark_image (GNOME_DRUID(info->druid),
+                                 "stock_split_watermark.png");
 
   /* account list */
   {
     GtkCList *clist;
 
-    info->account_list = lookup_widget (info->window, "account_clist");
+    info->account_list = glade_xml_get_widget (xml, "account_clist");
 
     clist = GTK_CLIST (info->account_list);
 
@@ -540,7 +544,7 @@ gnc_stock_split_druid_create (StockSplitInfo *info)
     gtk_signal_connect (GTK_OBJECT (clist), "select_row",
                         GTK_SIGNAL_FUNC (clist_select_row), info);
 
-    page = lookup_widget (info->window, "account_page");
+    page = glade_xml_get_widget (xml, "account_page");
 
     gtk_signal_connect (GTK_OBJECT (page), "next",
                         GTK_SIGNAL_FUNC (account_next), info);
@@ -553,20 +557,19 @@ gnc_stock_split_druid_create (StockSplitInfo *info)
     GtkWidget *date;
     GtkWidget *ce;
 
-    info->description_entry =
-      lookup_widget (info->window, "description_entry");
+    info->description_entry = glade_xml_get_widget (xml, "description_entry");
 
-    box = lookup_widget (info->window, "date_box");
+    box = glade_xml_get_widget (xml, "date_box");
     date = gnc_date_edit_new(time(NULL), FALSE, FALSE);
     gtk_box_pack_start (GTK_BOX (box), date, TRUE, TRUE, 0);
     info->date_edit = date;
 
-    box = lookup_widget (info->window, "distribution_box");
+    box = glade_xml_get_widget (xml, "distribution_box");
     amount = gnc_amount_edit_new ();
     gtk_box_pack_start (GTK_BOX (box), amount, TRUE, TRUE, 0);
     info->distribution_edit = amount;
 
-    box = lookup_widget (info->window, "price_box");
+    box = glade_xml_get_widget (xml, "price_box");
     amount = gnc_amount_edit_new ();
     gnc_amount_edit_set_print_info (GNC_AMOUNT_EDIT (amount),
                                     gnc_default_price_print_info ());
@@ -574,13 +577,15 @@ gnc_stock_split_druid_create (StockSplitInfo *info)
     gtk_box_pack_start (GTK_BOX (box), amount, TRUE, TRUE, 0);
     info->price_edit = amount;
 
-    box = lookup_widget (info->window, "price_currency_box");
+    box = glade_xml_get_widget (xml, "price_currency_box");
     ce = gnc_currency_edit_new ();
     gtk_box_pack_start (GTK_BOX (box), ce, TRUE, TRUE, 0);
     info->price_currency_edit = ce;
 
-    page = lookup_widget (info->window, "details_page");
+    page = glade_xml_get_widget (xml, "details_page");
 
+    gtk_signal_connect (GTK_OBJECT (page), "prepare",
+                        GTK_SIGNAL_FUNC (details_prepare), info);
     gtk_signal_connect (GTK_OBJECT (page), "next",
                         GTK_SIGNAL_FUNC (details_next), info);
   }
@@ -594,12 +599,12 @@ gnc_stock_split_druid_create (StockSplitInfo *info)
     GtkWidget *amount;
     GtkWidget *scroll;
 
-    box = lookup_widget (info->window, "cash_box");
+    box = glade_xml_get_widget (xml, "cash_box");
     amount = gnc_amount_edit_new ();
     gtk_box_pack_start (GTK_BOX (box), amount, TRUE, TRUE, 0);
     info->cash_edit = amount;
 
-    info->memo_entry = lookup_widget (info->window, "memo_entry");
+    info->memo_entry = glade_xml_get_widget (xml, "memo_entry");
 
     /* income tree */
     tree = gnc_account_tree_new ();
@@ -615,13 +620,9 @@ gnc_stock_split_druid_create (StockSplitInfo *info)
 
     gnc_account_tree_set_view_info (GNC_ACCOUNT_TREE (tree), &view_info);
 
-    gnc_account_tree_set_selectable_filter (GNC_ACCOUNT_TREE (tree),
-                                            account_currency_filter,
-                                            info);
-
     gtk_widget_show (tree);
 
-    scroll = lookup_widget (info->window, "income_scroll");
+    scroll = glade_xml_get_widget (xml, "income_scroll");
     gtk_container_add (GTK_CONTAINER (scroll), tree);
 
 
@@ -640,16 +641,12 @@ gnc_stock_split_druid_create (StockSplitInfo *info)
 
     gnc_account_tree_set_view_info (GNC_ACCOUNT_TREE (tree), &view_info);
 
-    gnc_account_tree_set_selectable_filter (GNC_ACCOUNT_TREE (tree),
-                                            account_currency_filter,
-                                            info);
-
     gtk_widget_show (tree);
 
-    scroll = lookup_widget (info->window, "asset_scroll");
+    scroll = glade_xml_get_widget (xml, "asset_scroll");
     gtk_container_add (GTK_CONTAINER (scroll), tree);
 
-    page = lookup_widget (info->window, "cash_page");
+    page = glade_xml_get_widget (xml, "cash_page");
 
     gtk_signal_connect (GTK_OBJECT (page), "prepare",
                         GTK_SIGNAL_FUNC (cash_prepare), info);
@@ -658,7 +655,7 @@ gnc_stock_split_druid_create (StockSplitInfo *info)
                         GTK_SIGNAL_FUNC (cash_next), info);
   }
 
-  page = lookup_widget (info->window, "finish_page");
+  page = glade_xml_get_widget (xml, "finish_page");
 
   gtk_signal_connect (GTK_OBJECT (page), "finish",
                       GTK_SIGNAL_FUNC (stock_split_finish), info);
@@ -672,9 +669,10 @@ refresh_handler (GHashTable *changes, gpointer user_data)
   Account *new_account;
   GNCIdType id_type;
   GtkWidget *page;
+  GladeXML *xml;
 
-  id_type = xaccGUIDType (&info->account);
-  old_account = xaccAccountLookup (&info->account);
+  id_type = xaccGUIDType (&info->account, gnc_get_current_book ());
+  old_account = xaccAccountLookup (&info->account, gnc_get_current_book ());
 
   if (fill_account_list (info, old_account) == 0)
   {
@@ -682,12 +680,13 @@ refresh_handler (GHashTable *changes, gpointer user_data)
     return;
   }
 
-  new_account = xaccAccountLookup (&info->account);
+  new_account = xaccAccountLookup (&info->account, gnc_get_current_book ());
 
-  if (id_type == GNC_ID_NULL || old_account == new_account)
+  if (!safe_strcmp (id_type, GNC_ID_NULL) || old_account == new_account)
     return;
 
-  page = lookup_widget (info->window, "account_page");
+  xml = glade_get_widget_tree (info->window);
+  page = glade_xml_get_widget (xml, "account_page");
 
   gnome_druid_set_page (GNOME_DRUID (info->druid), GNOME_DRUID_PAGE (page));
 }
@@ -734,6 +733,7 @@ gnc_stock_split_dialog (Account * initial)
     return;
   }
 
+  gnome_window_icon_set_from_default(GTK_WINDOW(info->window));
   gtk_widget_show_all (info->window);
 
   gnc_window_adjust_for_screen (GTK_WINDOW(info->window));
