@@ -26,14 +26,15 @@
 
 #include <gnome.h>
 
-#include "FileDialog.h"
 #include "date.h"
 #include "dialog-utils.h"
 #include "global-options.h"
 #include "gnc-ui-util.h"
 #include "messages.h"
 #include "reconcile-list.h"
-
+#include "Transaction.h"
+#include "search-param.h"
+#include "gnc-component-manager.h"
 
 /* Signal codes */
 enum
@@ -45,19 +46,20 @@ enum
 
 
 /** Static Globals ****************************************************/
-static GtkCListClass *parent_class = NULL;
+static GNCQueryListClass *parent_class = NULL;
 static guint reconcile_list_signals[LAST_SIGNAL] = {0};
 
 
 /** Static function declarations **************************************/
 static void gnc_reconcile_list_init(GNCReconcileList *list);
 static void gnc_reconcile_list_class_init(GNCReconcileListClass *klass);
-static void gnc_reconcile_list_select_row(GtkCList *clist, gint row,
-					  gint column, GdkEvent *event);
-static void gnc_reconcile_list_unselect_row(GtkCList *clist, gint row,
-					    gint column, GdkEvent *event);
 static void gnc_reconcile_list_destroy(GtkObject *object);
-static void gnc_reconcile_list_fill(GNCReconcileList *list);
+static gpointer gnc_reconcile_list_is_reconciled(gpointer item, gpointer user_data);
+static void gnc_reconcile_list_line_toggled (GNCQueryList *list, gpointer item,
+					     gpointer user_data);
+static void gnc_reconcile_list_double_click_entry (GNCQueryList *list,
+						   gpointer item,
+						   gpointer user_data);
 
 
 GtkType
@@ -79,7 +81,7 @@ gnc_reconcile_list_get_type (void)
       (GtkClassInitFunc) NULL
     };
 
-    gnc_reconcile_list_type = gtk_type_unique(GTK_TYPE_CLIST,
+    gnc_reconcile_list_type = gtk_type_unique(GTK_TYPE_GNC_QUERY_LIST,
 					      &gnc_reconcile_list_info);
   }
 
@@ -95,117 +97,130 @@ gnc_reconcile_list_get_type (void)
  *       type    - the type of list, RECLIST_DEBIT or RECLIST_CREDIT*
  * Returns: the account tree widget, or NULL if there was a problem.*
 \********************************************************************/
+static void
+gnc_reconcile_list_construct(GNCReconcileList *list, Query *query)
+{
+  GNCQueryList *qlist = GNC_QUERY_LIST (list);
+  gboolean inv_sort = FALSE;
+
+  if (list->list_type == RECLIST_CREDIT)
+    inv_sort = TRUE;
+
+  gnc_query_list_construct (qlist, list->column_list, query);
+  gnc_query_list_set_numerics (qlist, TRUE, inv_sort);
+
+  /* Now set up the signals for the QueryList */
+  gtk_signal_connect (GTK_OBJECT (qlist), "line_toggled",
+		      gnc_reconcile_list_line_toggled, list);
+  gtk_signal_connect (GTK_OBJECT (qlist), "double_click_entry",
+		      gnc_reconcile_list_double_click_entry, list);
+}
+
 GtkWidget *
 gnc_reconcile_list_new(Account *account, GNCReconcileListType type)
 {
   GNCReconcileList *list;
-  gboolean include_children;
+  gboolean include_children, auto_check;
+  GList *accounts = NULL;
+  GList *splits;
+  Query *query;
 
-  g_assert(account != NULL);
-  g_assert((type == RECLIST_DEBIT) || (type == RECLIST_CREDIT));
+  g_return_val_if_fail(account, NULL);
+  g_return_val_if_fail((type == RECLIST_DEBIT) ||
+                       (type == RECLIST_CREDIT), NULL);
 
   list = GNC_RECONCILE_LIST(gtk_type_new(gnc_reconcile_list_get_type()));
 
   list->account = account;
   list->list_type = type;
 
-  list->query = xaccMallocQuery();
-
-  xaccQuerySetGroup(list->query, gncGetCurrentGroup());
-
-  /* match the account */
-  xaccQueryAddSingleAccountMatch(list->query, account, QUERY_OR);
+  query = xaccMallocQuery();
+  xaccQuerySetBook(query, gnc_get_current_book ());
 
   include_children = xaccAccountGetReconcileChildrenStatus(account);
-  if(include_children)
-  {
-    /* match child accounts */
-    AccountGroup *account_group = xaccAccountGetChildren(account);
-    GList *children_list = xaccGroupGetSubAccounts(account_group);
-    GList *node;
+  if (include_children)
+    accounts = xaccAccountGetDescendants(account);
 
-    for (node = children_list; node; node = node->next)
-      xaccQueryAddSingleAccountMatch (list->query, node->data, QUERY_OR);
+  /* match the account */
+  accounts = g_list_prepend (accounts, account);
 
-    g_list_free (children_list);
+  xaccQueryAddAccountMatch (query, accounts, GUID_MATCH_ANY, QUERY_AND);
+
+  g_list_free (accounts);
+
+  /* limit the matches to CREDITs and DEBITs only, depending on the type */
+  if (type == RECLIST_CREDIT)
+    xaccQueryAddValueMatch(query, gnc_numeric_zero (),
+			   NUMERIC_MATCH_CREDIT,
+			   COMPARE_GTE, QUERY_AND);
+  else
+    xaccQueryAddValueMatch(query, gnc_numeric_zero (),
+			   NUMERIC_MATCH_DEBIT,
+			   COMPARE_GTE, QUERY_AND);
+
+  /* limit the matches only to Cleared and Non-reconciled splits */
+  xaccQueryAddClearedMatch(query, CLEARED_NO | CLEARED_CLEARED, QUERY_AND);
+
+  /* initialize the QueryList */
+  gnc_reconcile_list_construct (list, query);
+
+  /* find the list of splits to auto-reconcile */
+  auto_check = gnc_lookup_boolean_option ("Reconcile",
+                                          "Check off cleared transactions",
+                                          TRUE);
+
+  if (auto_check) {
+    for (splits = xaccQueryGetSplits(query); splits; splits = splits->next) {
+      Split *split = splits->data;
+      char recn = xaccSplitGetReconcile(split);
+    
+      /* Just an extra verification that our query is correct ;) */
+      g_assert (recn == NREC || recn == CREC);
+
+      if (recn == CREC)
+	g_hash_table_insert (list->reconciled, split, split);
+    }
   }
 
-  if (type == RECLIST_CREDIT)
-    DxaccQueryAddAmountMatch(list->query, 0.0, AMT_SGN_MATCH_CREDIT,
-                             AMT_MATCH_ATLEAST, QUERY_AND);
-  else
-    DxaccQueryAddAmountMatch(list->query, 0.0, AMT_SGN_MATCH_DEBIT,
-                             AMT_MATCH_ATLEAST, QUERY_AND);
+  /* Free the query -- we don't need it anymore */
+  xaccFreeQuery(query);
 
   return GTK_WIDGET(list);
 }
 
-static void
-update_toggle (GtkCList *list, gint row)
-{
-  GNCReconcileList *rlist = GNC_RECONCILE_LIST (list);
-  gboolean reconciled;
-  Split *split;
-
-  split = gtk_clist_get_row_data (list, row);
-  reconciled = g_hash_table_lookup (rlist->reconciled, split) != NULL;
-
-  gnc_clist_set_check (list, row, 4, reconciled);
-}
-
 static void
 gnc_reconcile_list_init (GNCReconcileList *list)
 {
-  GtkCList *clist = GTK_CLIST (list);
-  GtkStyle *style;
-  gchar * titles[] =
-    {
-      _("Date"),
-      _("Num"),
-      _("Description"),
-      _("Amount"),
-      _("Reconciled:R") + 11,
-      NULL
-    };
+  GNCSearchParam *param;
+  GList *columns = NULL;
 
-  list->num_splits = 0;
-  list->num_columns = 0;
   list->reconciled = g_hash_table_new (NULL, NULL);
-  list->current_row = -1;
-  list->current_split = NULL;
-  list->no_toggle = FALSE;
-  list->always_unselect = FALSE;
-  list->first_fill = TRUE;
-  list->query = NULL;
+  list->account = NULL;
+  list->sibling = NULL;
 
-  while (titles[list->num_columns] != NULL)
-    list->num_columns++;
+  param = gnc_search_param_new();
+  gnc_search_param_set_param_fcn (param, QUERYCORE_BOOLEAN,
+				  gnc_reconcile_list_is_reconciled, list);
+  gnc_search_param_set_title (param, _("Reconciled:R")+11);
+  gnc_search_param_set_justify (param, GTK_JUSTIFY_CENTER);
+  gnc_search_param_set_passive (param, TRUE);
+  gnc_search_param_set_non_resizeable (param, TRUE);
+  columns = g_list_prepend (columns, param);
+  columns = gnc_search_param_prepend_with_justify (columns, _("Amount"),
+						   GTK_JUSTIFY_RIGHT,
+						   NULL, GNC_ID_SPLIT,
+						   SPLIT_AMOUNT, NULL);
+  columns = gnc_search_param_prepend (columns, _("Description"), NULL,
+				      GNC_ID_SPLIT, SPLIT_TRANS,
+				      TRANS_DESCRIPTION, NULL);
+  columns = gnc_search_param_prepend_with_justify (columns, _("Num"),
+						   GTK_JUSTIFY_CENTER,
+						   NULL, GNC_ID_SPLIT,
+						   SPLIT_TRANS, TRANS_NUM, NULL);
+  columns = gnc_search_param_prepend (columns, _("Date"), NULL, GNC_ID_SPLIT,
+				      SPLIT_TRANS, TRANS_DATE_POSTED, NULL);
 
-  gtk_clist_construct (clist, list->num_columns, titles);
-  gtk_clist_set_shadow_type (clist, GTK_SHADOW_IN);
-  gtk_clist_set_column_justification (clist, 3, GTK_JUSTIFY_RIGHT);
-  gtk_clist_set_column_justification (clist, 4, GTK_JUSTIFY_CENTER);
-  gtk_clist_column_titles_passive (clist);
-
-  style = gtk_widget_get_style (GTK_WIDGET(list));
-
-  {
-    GdkFont *font = NULL;
-    gint width;
-    gint i;
-
-    font = style->font;
-    if (font != NULL)
-    {
-      for (i = 0; i < list->num_columns; i++)
-      {
-	width = gdk_string_width (font, titles[i]);
-	gtk_clist_set_column_min_width (clist, i, width + 5);
-	if (i == 4)
-	  gtk_clist_set_column_max_width (clist, i, width + 5);
-      }
-    }
-  }
+  list->column_list = columns;
 }
 
 
@@ -215,14 +230,12 @@ gnc_reconcile_list_class_init (GNCReconcileListClass *klass)
   GtkObjectClass    *object_class;
   GtkWidgetClass    *widget_class;
   GtkContainerClass *container_class;
-  GtkCListClass     *clist_class;
 
   object_class =    (GtkObjectClass*) klass;
   widget_class =    (GtkWidgetClass*) klass;
   container_class = (GtkContainerClass*) klass;
-  clist_class =     (GtkCListClass*) klass;
 
-  parent_class = gtk_type_class(GTK_TYPE_CLIST);
+  parent_class = gtk_type_class(GTK_TYPE_GNC_QUERY_LIST);
 
   reconcile_list_signals[TOGGLE_RECONCILED] =
     gtk_signal_new("toggle_reconciled",
@@ -250,35 +263,26 @@ gnc_reconcile_list_class_init (GNCReconcileListClass *klass)
 
   object_class->destroy = gnc_reconcile_list_destroy;
 
-  clist_class->select_row = gnc_reconcile_list_select_row;
-  clist_class->unselect_row = gnc_reconcile_list_unselect_row;
-
   klass->toggle_reconciled = NULL;
   klass->double_click_split = NULL;
 }
 
 static void
-gnc_reconcile_list_toggle_row(GNCReconcileList *list, gint row)
+gnc_reconcile_list_toggle_split(GNCReconcileList *list, Split *split)
 {
-  Split *split, *current;
+  Split *current;
 
-  g_assert (IS_GNC_RECONCILE_LIST(list));
-  g_assert (list->reconciled != NULL);
+  g_return_if_fail (IS_GNC_RECONCILE_LIST(list));
+  g_return_if_fail (list->reconciled != NULL);
 
-  if (list->no_toggle)
-    return;
-
-  split = gtk_clist_get_row_data (GTK_CLIST(list), row);
   current = g_hash_table_lookup (list->reconciled, split);
 
-  list->current_split = split;
-
   if (current == NULL)
     g_hash_table_insert (list->reconciled, split, split);
   else
     g_hash_table_remove (list->reconciled, split);
 
-  update_toggle (GTK_CLIST (list), row);
+  gnc_query_list_refresh_item (GNC_QUERY_LIST(list), split);
 }
 
 static void
@@ -302,63 +306,40 @@ gnc_reconcile_list_toggle_children(Account *account, GNCReconcileList *list, Spl
   for(node = xaccTransGetSplitList(transaction); node; node = node->next)
   {
     Split *other_split;
-    Transaction *other_transaction;
     Account *other_account;
     GNCReconcileList *current_list;
-    gint row;
-    GtkCList *split_list;
 
     other_split = node->data;
-    other_transaction = xaccSplitGetParent(other_split);
     other_account = xaccSplitGetAccount(other_split);
     if(other_split == split)
       continue;
     /* Check this 'other' account in in the same heirarchy */
     if(!g_list_find(child_accounts,other_account))
       continue;
-    /* Search our sibling list for this spliti first.  We search the 
+    /* Search our sibling list for this split first.  We search the 
      * sibling list first because that it where it is most likely to be.
      */
     current_list = list->sibling;
-    row = gtk_clist_find_row_from_data(GTK_CLIST(&current_list->clist), other_split);
-    if(row == -1) {
+    if (!gnc_query_list_item_in_list (GNC_QUERY_LIST(current_list), other_split)) {
       /* Not in the sibling list, try this list */
       current_list = list;
-      row = gtk_clist_find_row_from_data(GTK_CLIST(&current_list->clist), other_split);
-      if(row == -1)
+      if (!gnc_query_list_item_in_list (GNC_QUERY_LIST(current_list), other_split))
 	/* We can't find it, nothing more I can do about it */
 	continue;
     }
-    gnc_reconcile_list_toggle_row(current_list, row);
+    gnc_reconcile_list_toggle_split(current_list, other_split);
   }
 }
 
 static void
-gnc_reconcile_list_toggle (GNCReconcileList *list)
+gnc_reconcile_list_toggle (GNCReconcileList *list, Split *split)
 {
-  Split *split, *current;
-  gint row;
-  Account * account;
   gboolean include_children;
 
-  g_assert (IS_GNC_RECONCILE_LIST(list));
-  g_assert (list->reconciled != NULL);
+  g_return_if_fail (IS_GNC_RECONCILE_LIST(list));
+  g_return_if_fail (list->reconciled != NULL);
 
-  if (list->no_toggle)
-    return;
-
-  row = list->current_row;
-  split = gtk_clist_get_row_data (GTK_CLIST(list), row);
-  current = g_hash_table_lookup (list->reconciled, split);
-
-  list->current_split = split;
-
-  if (current == NULL)
-    g_hash_table_insert (list->reconciled, split, split);
-  else
-    g_hash_table_remove (list->reconciled, split);
-
-  update_toggle (GTK_CLIST (list), row);
+  gnc_reconcile_list_toggle_split(list, split);
 
   include_children = xaccAccountGetReconcileChildrenStatus(list->account);
   if(include_children)
@@ -369,52 +350,30 @@ gnc_reconcile_list_toggle (GNCReconcileList *list)
 }
 
 static void
-gnc_reconcile_list_select_row (GtkCList *clist, gint row, gint column,
-			       GdkEvent *event)
+gnc_reconcile_list_line_toggled (GNCQueryList *list, gpointer item,
+				 gpointer user_data)
 {
-  GNCReconcileList *list = GNC_RECONCILE_LIST(clist);
+  GNCReconcileList *rlist = user_data;
 
-  list->current_row = row;
+  g_return_if_fail(item);
+  g_return_if_fail(user_data);
+  g_return_if_fail(IS_GNC_RECONCILE_LIST(rlist));
 
-  gnc_reconcile_list_toggle (list);
-
-  GTK_CLIST_CLASS(parent_class)->select_row (clist, row, column, event);
-
-  if (event && (event->type == GDK_2BUTTON_PRESS))
-  {
-    Split *split;
-
-    split = gtk_clist_get_row_data (clist, row);
-
-    gtk_signal_emit(GTK_OBJECT(list),
-                    reconcile_list_signals[DOUBLE_CLICK_SPLIT], split);
-  }
+  gnc_reconcile_list_toggle (rlist, item);
 }
 
-static void
-gnc_reconcile_list_unselect_row (GtkCList *clist, gint row, gint column,
-                                 GdkEvent *event)
+static void gnc_reconcile_list_double_click_entry (GNCQueryList *list,
+						   gpointer item,
+						   gpointer user_data)
 {
-  GNCReconcileList *list = GNC_RECONCILE_LIST(clist);
+  GNCReconcileList *rlist = user_data;
 
-  if (row == list->current_row)
-  {
-    gnc_reconcile_list_toggle (list);
-    if (!list->always_unselect)
-      return;
-  }
+  g_return_if_fail(item);
+  g_return_if_fail(user_data);
+  g_return_if_fail(IS_GNC_RECONCILE_LIST(rlist));
 
-  GTK_CLIST_CLASS(parent_class)->unselect_row (clist, row, column, event);
-
-  if (event && (event->type == GDK_2BUTTON_PRESS))
-  {
-    Split *split;
-
-    split = gtk_clist_get_row_data (clist, row);
-
-    gtk_signal_emit (GTK_OBJECT(list),
-                     reconcile_list_signals[DOUBLE_CLICK_SPLIT], split);
-  }
+  gtk_signal_emit(GTK_OBJECT(rlist),
+		  reconcile_list_signals[DOUBLE_CLICK_SPLIT], item);
 }
 
 static void
@@ -422,18 +381,24 @@ gnc_reconcile_list_destroy (GtkObject *object)
 {
   GNCReconcileList *list = GNC_RECONCILE_LIST(object);
 
+  if (list->column_list)
+  {
+    GList *node;
+    for (node = list->column_list; node; node = node->next)
+    {
+      GNCSearchParam *param = node->data;
+      gtk_object_destroy (GTK_OBJECT(param));
+    }
+    g_list_free(list->column_list);
+    list->column_list = NULL;
+  }
+
   if (list->reconciled != NULL)
   {
     g_hash_table_destroy(list->reconciled);
     list->reconciled = NULL;
   }
 
-  if (list->query != NULL)
-  {
-    xaccFreeQuery(list->query);
-    list->query = NULL;
-  }
-
   if (GTK_OBJECT_CLASS(parent_class)->destroy)
     GTK_OBJECT_CLASS(parent_class)->destroy (object);
 }
@@ -441,25 +406,13 @@ gnc_reconcile_list_destroy (GtkObject *object)
 gint
 gnc_reconcile_list_get_needed_height (GNCReconcileList *list, gint num_rows)
 {
-  GtkCList *clist;
-  gint list_height;
-  gint title_height;
-
   g_return_val_if_fail (list != NULL, 0);
   g_return_val_if_fail (IS_GNC_RECONCILE_LIST(list), 0);
 
   if (!GTK_WIDGET_REALIZED (list))
     return 0;
 
-  clist = GTK_CLIST (list);
-
-  /* sync with gtkclist.c */
-  title_height = (clist->column_title_area.height +
-                  (GTK_WIDGET(list)->style->klass->ythickness +
-                   GTK_CONTAINER(list)->border_width) * 2);
-  list_height = (clist->row_height * num_rows) + (num_rows + 1);
-
-  return title_height + list_height;
+  return gnc_query_list_get_needed_height (GNC_QUERY_LIST(list), num_rows);
 }
 
 gint
@@ -468,7 +421,7 @@ gnc_reconcile_list_get_num_splits (GNCReconcileList *list)
   g_return_val_if_fail (list != NULL, 0);
   g_return_val_if_fail (IS_GNC_RECONCILE_LIST(list), 0);
 
-  return list->num_splits;
+  return gnc_query_list_get_num_entries(GNC_QUERY_LIST(list));
 }
 
 Split *
@@ -477,7 +430,42 @@ gnc_reconcile_list_get_current_split (GNCReconcileList *list)
   g_return_val_if_fail (list != NULL, NULL);
   g_return_val_if_fail (IS_GNC_RECONCILE_LIST(list), NULL);
 
-  return list->current_split;
+  return gnc_query_list_get_current_entry(GNC_QUERY_LIST(list));
+}
+
+/********************************************************************\
+ * gnc_reconcile_list_is_reconciled                                 *
+ *   Is the item a reconciled split?                                *
+ *                                                                  *
+ * Args: item - the split to be checked                             *
+ *       user_data - a pointer to the GNCReconcileList              *
+ * Returns: whether the split is to be reconciled.                  *
+\********************************************************************/
+static gpointer
+gnc_reconcile_list_is_reconciled(gpointer item, gpointer user_data)
+{
+  GNCReconcileList *list = user_data;
+  Split *current;
+
+  g_return_val_if_fail(item, NULL);
+  g_return_val_if_fail(list, NULL);
+  g_return_val_if_fail(IS_GNC_RECONCILE_LIST(list), NULL);
+
+  if (!list->reconciled)
+    return NULL;
+
+  current = g_hash_table_lookup(list->reconciled, item);
+  return GINT_TO_POINTER(current != NULL);
+}
+
+static void
+grl_refresh_helper (gpointer key, gpointer value, gpointer user_data)
+{
+  GNCReconcileList *list = user_data;
+  GNCQueryList *qlist = GNC_QUERY_LIST(list);
+
+  if (!gnc_query_list_item_in_list(qlist, key))
+    g_hash_table_remove(list->reconciled, key);
 }
 
 /********************************************************************\
@@ -490,67 +478,17 @@ gnc_reconcile_list_get_current_split (GNCReconcileList *list)
 void
 gnc_reconcile_list_refresh (GNCReconcileList *list)
 {
-  GtkCList *clist = GTK_CLIST(list);
-  GtkAdjustment *adjustment;
-  gfloat save_value = 0.0;
-  Split *old_focus_split;
-  Split *old_split;
-  gint old_focus_row;
-  gint new_row;
+  GNCQueryList *qlist;
 
   g_return_if_fail (list != NULL);
   g_return_if_fail (IS_GNC_RECONCILE_LIST(list));
 
-  adjustment = gtk_clist_get_vadjustment (GTK_CLIST(list));
-  if (adjustment != NULL)
-    save_value = adjustment->value;
+  qlist = GNC_QUERY_LIST(list);
+  gnc_query_list_refresh(qlist);
 
-  old_focus_row = clist->focus_row;
-  old_focus_split = gtk_clist_get_row_data (clist, old_focus_row);
-
-  gtk_clist_freeze (clist);
-
-  gtk_clist_clear (clist);
-
-  old_split = list->current_split;
-  list->num_splits = 0;
-  list->current_row = -1;
-  list->current_split = NULL;
-
-  gnc_reconcile_list_fill (list);
-
-  gtk_clist_columns_autosize (clist);
-
-  if (adjustment)
-  {
-    save_value = CLAMP (save_value, adjustment->lower, adjustment->upper);
-    gtk_adjustment_set_value (adjustment, save_value);
-  }
-
-  if (old_split)
-  {
-    new_row = gtk_clist_find_row_from_data (clist, old_split);
-    if (new_row >= 0)
-    {
-      list->no_toggle = TRUE;
-      gtk_clist_select_row (clist, new_row, 0);
-      list->no_toggle = FALSE;
-      list->current_split = old_split;
-    }
-  }
-
-  if (old_focus_split)
-  {
-    new_row = gtk_clist_find_row_from_data (clist, old_focus_split);
-
-    if (new_row < 0)
-      new_row = old_focus_row;
-
-    if (new_row >= 0)
-      clist->focus_row = new_row;
-  }
-
-  gtk_clist_thaw (clist);
+  /* Now verify that everything in the reconcile hash is still in qlist */
+  if (list->reconciled)
+    g_hash_table_foreach(list->reconciled, grl_refresh_helper, list);
 }
 
 
@@ -561,16 +499,19 @@ gnc_reconcile_list_refresh (GNCReconcileList *list)
  * Args: list - list to get reconciled balance of                   *
  * Returns: reconciled balance (gnc_numeric)                        *
 \********************************************************************/
+static void
+grl_balance_hash_helper (gpointer key, gpointer value, gpointer user_data)
+{
+  Split *split = key;
+  gnc_numeric *total = user_data;
+
+  *total = gnc_numeric_add_fixed (*total, xaccSplitGetAmount (split));
+}
+
 gnc_numeric
 gnc_reconcile_list_reconciled_balance (GNCReconcileList *list)
 {
-  GtkCList *clist = GTK_CLIST(list);
-  Split *split;
-  gnc_numeric total;
-  int account_type;
-  int i;
-
-  total = gnc_numeric_zero ();
+  gnc_numeric total = gnc_numeric_zero ();
 
   g_return_val_if_fail (list != NULL, total);
   g_return_val_if_fail (IS_GNC_RECONCILE_LIST(list), total);
@@ -578,21 +519,7 @@ gnc_reconcile_list_reconciled_balance (GNCReconcileList *list)
   if (list->reconciled == NULL)
     return total;
 
-  account_type = xaccAccountGetType (list->account);
-
-  for (i = 0; i < list->num_splits; i++)
-  {
-    split = gtk_clist_get_row_data (clist, i);
-
-    if (g_hash_table_lookup (list->reconciled, split) == NULL)
-      continue;
-
-    if ((account_type == STOCK) || (account_type == MUTUAL) ||
-        (account_type == CURRENCY))
-      total = gnc_numeric_add_fixed (total, xaccSplitGetShareAmount(split));
-    else
-      total = gnc_numeric_add_fixed (total, xaccSplitGetValue(split));
-  }
+  g_hash_table_foreach (list->reconciled, grl_balance_hash_helper, &total);
 
   return gnc_numeric_abs (total);
 }
@@ -600,47 +527,45 @@ gnc_reconcile_list_reconciled_balance (GNCReconcileList *list)
 
 /********************************************************************\
  * gnc_reconcile_list_commit                                        *
- *   commit the reconcile information in the list                   *
+ *   Commit the reconcile information in the list. only change the  *
+ *   state of those items marked as reconciled.  All others should  *
+ *   retain their previous state (none, cleared, voided, etc.).     *
  *                                                                  *
  * Args: list - list to commit                                      *
  *       date - date to set as the reconcile date                   *
  * Returns: nothing                                                 *
 \********************************************************************/
+static void
+grl_commit_hash_helper (gpointer key, gpointer value, gpointer user_data)
+{
+  Split *split = key;
+  time_t *date = user_data;
+  Transaction *trans;
+
+  trans = xaccSplitGetParent(split);
+  xaccTransBeginEdit(trans);
+  xaccSplitSetReconcile (split, YREC);
+  xaccSplitSetDateReconciledSecs (split, *date);
+  xaccTransCommitEdit(trans);
+}
+
 void
 gnc_reconcile_list_commit (GNCReconcileList *list, time_t date)
 {
-  GtkCList *clist = GTK_CLIST(list);
-  Split *split;
-  int i;
-
   g_return_if_fail (list != NULL);
   g_return_if_fail (IS_GNC_RECONCILE_LIST(list));
 
   if (list->reconciled == NULL)
     return;
 
-  for (i = 0; i < list->num_splits; i++)
-  {
-    Transaction *trans;
-    char recn;
-
-    split = gtk_clist_get_row_data (clist, i);
-    trans = xaccSplitGetParent(split);
-
-    recn = g_hash_table_lookup (list->reconciled, split) ? YREC : NREC;
-
-    xaccTransBeginEdit(trans);
-    xaccSplitSetReconcile (split, recn);
-    if (recn == YREC) {
-      xaccSplitSetDateReconciledSecs (split, date);
-    }
-    xaccTransCommitEdit(trans);
-  }
+  gnc_suspend_gui_refresh();
+  g_hash_table_foreach (list->reconciled, grl_commit_hash_helper, &date);
+  gnc_resume_gui_refresh();
 }
 
 
 /********************************************************************\
- * gnc_reconcile_list_commit                                        *
+ * gnc_reconcile_list_postpone                                      *
  *   postpone the reconcile information in the list by setting      *
  *   reconciled splits to cleared status                            *
  *                                                                  *
@@ -651,8 +576,9 @@ void
 gnc_reconcile_list_postpone (GNCReconcileList *list)
 {
   Transaction *trans;
-  GtkCList *clist = GTK_CLIST(list);
+  GtkCList *clist = GTK_CLIST(list); /* This is cheating! */
   Split *split;
+  int num_splits;
   int i;
 
   g_return_if_fail(list != NULL);
@@ -661,7 +587,9 @@ gnc_reconcile_list_postpone (GNCReconcileList *list)
   if (list->reconciled == NULL)
     return;
 
-  for (i = 0; i < list->num_splits; i++)
+  num_splits = gnc_query_list_get_num_entries(GNC_QUERY_LIST(list));
+  gnc_suspend_gui_refresh();
+  for (i = 0; i < num_splits; i++)
   {
     char recn;
 
@@ -674,6 +602,7 @@ gnc_reconcile_list_postpone (GNCReconcileList *list)
     xaccSplitSetReconcile (split, recn);
     xaccTransCommitEdit(trans);
   }
+  gnc_resume_gui_refresh();
 }
 
 
@@ -690,15 +619,7 @@ gnc_reconcile_list_unselect_all(GNCReconcileList *list)
   g_return_if_fail (list != NULL);
   g_return_if_fail (IS_GNC_RECONCILE_LIST(list));
 
-  list->no_toggle = TRUE;
-  list->always_unselect = TRUE;
-
-  gtk_clist_unselect_all (GTK_CLIST(list));
-
-  list->always_unselect = FALSE;
-  list->no_toggle = FALSE;
-
-  list->current_split = NULL;
+  gnc_query_list_unselect_all (GNC_QUERY_LIST(list));
 }
 
 
@@ -718,40 +639,12 @@ gnc_reconcile_list_changed (GNCReconcileList *list)
   return g_hash_table_size (list->reconciled) != 0;
 }
 
-
-/********************************************************************\
- * gnc_reconcile_list_set_sort_order                                *
- *   sets the sorting order of splits in the list                   *
- *                                                                  *
- * Args: list - list to change the sort order for                   *
- * Returns: nothing                                                 *
-\********************************************************************/
-void
-gnc_reconcile_list_set_sort_order (GNCReconcileList *list, sort_type_t key)
-{
-  g_return_if_fail (list != NULL);
-  g_return_if_fail (IS_GNC_RECONCILE_LIST(list));
-  g_return_if_fail (list->query != NULL);
-
-  xaccQuerySetSortOrder (list->query, key,
-                         (key == BY_STANDARD) ? BY_NONE : BY_STANDARD,
-                         BY_NONE);
-
-  if (list->list_type == RECLIST_DEBIT)
-    return;
-
-  xaccQuerySetSortIncreasing (list->query,
-                              !(key == BY_AMOUNT),
-                              !(key == BY_AMOUNT),
-                              !(key == BY_AMOUNT));
-}
-
+#if 0
 static void
 gnc_reconcile_list_fill(GNCReconcileList *list)
 {
   const gchar *strings[list->num_columns + 1];
   GNCPrintAmountInfo print_info;
-  GNCAccountType account_type;
   Transaction *trans;
   gboolean auto_check;
   GList *splits;
@@ -761,45 +654,47 @@ gnc_reconcile_list_fill(GNCReconcileList *list)
                                           "Check off cleared transactions",
                                           TRUE);
 
-  account_type = xaccAccountGetType (list->account);
   strings[5] = NULL;
 
-  if ((account_type == STOCK) ||
-      (account_type == MUTUAL) ||
-      (account_type == CURRENCY))
-    print_info = gnc_account_quantity_print_info (list->account, FALSE);
-  else
-    print_info = gnc_account_value_print_info (list->account, FALSE);
+  print_info = gnc_account_print_info (list->account, FALSE);
 
   for (splits = xaccQueryGetSplits (list->query); splits; splits=splits->next)
   {
     gnc_numeric amount;
     Timespec ts;
+    const gchar *trans_num;
     char recn;
-    int row;
+    int row, len;
 
     split = splits->data;
+    trans = xaccSplitGetParent (split);
+    trans_num = xaccTransGetNum (trans);
 
     recn = xaccSplitGetReconcile(split);
     if ((recn != NREC) && (recn != CREC))
       continue;
 
-    if((account_type == STOCK) || (account_type == MUTUAL))
-      amount = xaccSplitGetShareAmount (split);
-    else
-      amount = xaccSplitGetValue (split);
+    amount = xaccSplitGetAmount (split);
 
-    if (gnc_numeric_negative_p (amount) && list->list_type == RECLIST_DEBIT)
-      continue;
-    if (!gnc_numeric_negative_p (amount) && list->list_type == RECLIST_CREDIT)
-      continue;
+    if (gnc_numeric_negative_p (amount)) {
+      if (list->list_type == RECLIST_DEBIT) {
+	continue;
+      }
+    } else if (gnc_numeric_positive_p (amount)) {
+      if (list->list_type == RECLIST_CREDIT) {
+	continue;
+      }
+    } else {
+      len = trans_num ? strlen(trans_num) : 0;
+      if ((len  && (list->list_type == RECLIST_DEBIT)) ||
+	  (!len && (list->list_type == RECLIST_CREDIT)))
+	continue;
+    }
 
-    trans = xaccSplitGetParent (split);
-
-    xaccTransGetDateTS (trans, &ts);
+    xaccTransGetDatePostedTS (trans, &ts);
 
     strings[0] = gnc_print_date (ts);
-    strings[1] = xaccTransGetNum (trans);
+    strings[1] = trans_num;
     strings[2] = xaccTransGetDescription (trans);
     strings[3] = xaccPrintAmount (gnc_numeric_abs (amount), print_info);
     strings[4] = "";
@@ -817,3 +712,4 @@ gnc_reconcile_list_fill(GNCReconcileList *list)
 
   list->first_fill = FALSE;
 }
+#endif
