@@ -28,64 +28,54 @@
 #include <guile/gh.h>
 #include <popt.h>
 #include <stdlib.h>
-#ifdef GTKHTML_HAVE_GCONF
-#include <gconf/gconf.h>
-#endif
-
+#include <g-wrap-wct.h>
 #include <X11/Xlib.h>
 
 #include "AccWindow.h"
-#include "FileBox.h"
-#include "FileDialog.h"
-#include "SplitLedger.h"
 #include "TransLog.h"
-#include "argv-list-converters.h"
+#include "backend/gnc-backend-api.h"
 #include "combocell.h"
 #include "date.h"
-#include "dialog-account.h"
+#include "dialog-commodity.h"
+#include "dialog-options.h"
 #include "dialog-transfer.h"
 #include "dialog-utils.h"
-#include "extensions.h"
+#include "file-utils.h"
 #include "global-options.h"
 #include "gnc-component-manager.h"
 #include "gnc-engine-util.h"
-#include "gnc-splash.h"
+#include "gnc-file.h"
+#include "gnc-menu-extensions.h"
 #include "gnc-network.h"
-#ifdef USE_GUPPI
-#include "gnc-html-guppi.h"
-#endif
+#include "gnc-splash.h"
+#include "gnc-html.h"
+#include "gnc-gnome-utils.h"
 #include "gnc-gpg.h"
+#include "gnc-report.h"
+#include "gnc-split-reg.h"
 #include "gnc-ui.h"
-#include "gnc.h"
 #include "gnucash-color.h"
 #include "gnucash-sheet.h"
 #include "gnucash-style.h"
-#include "gnucash.h"
 #include "guile-util.h"
 #include "messages.h"
-#include "recncell.h"
-#include "splitreg.h"
+#include "split-register.h"
 #include "top-level.h"
 #include "window-help.h"
 #include "window-main.h"
 #include "window-acct-tree.h"
+#include "window-register.h"
 #include "window-report.h"
-#include "new-user-interface.h"
-#include "new-user-funs.h"
 
-#include <g-wrap-runtime-guile.h>
 
 /** PROTOTYPES ******************************************************/
 static void gnc_configure_date_format_cb(gpointer);
 static void gnc_configure_date_format(void);
 static void gnc_configure_account_separator_cb(gpointer);
-static void gnc_configure_account_separator(void);
 static void gnc_configure_register_colors_cb(gpointer);
 static void gnc_configure_register_colors(void);
 static void gnc_configure_register_borders_cb(gpointer);
 static void gnc_configure_register_borders(void);
-static void gnc_configure_reverse_balance_cb(gpointer);
-static void gnc_configure_reverse_balance(void);
 static void gnc_configure_auto_raise_cb(gpointer);
 static void gnc_configure_auto_raise(void);
 static void gnc_configure_negative_color_cb(gpointer);
@@ -94,6 +84,10 @@ static void gnc_configure_auto_decimal_cb(gpointer);
 static void gnc_configure_auto_decimal(void);
 static void gnc_configure_auto_decimal_places_cb(gpointer);
 static void gnc_configure_auto_decimal_places(void);
+static void gnc_configure_file_be_retention_days_cb(gpointer);
+static void gnc_configure_file_be_retention_days(void);
+static void gnc_configure_file_be_compression_cb(gpointer);
+static void gnc_configure_file_be_compression(void);
 static void gnc_configure_register_font_cb(gpointer);
 static void gnc_configure_register_font(void);
 static void gnc_configure_register_hint_font_cb(gpointer);
@@ -104,9 +98,8 @@ static void gnc_configure_register_hint_font(void);
 /* This static indicates the debugging module that this .o belongs to.  */
 static short module = MOD_GUI;
 
-static GNCMainInfo * app = NULL;
-
 static int gnome_is_running = FALSE;
+static int splash_is_initialized = FALSE;
 static int gnome_is_initialized = FALSE;
 static int gnome_is_terminating = FALSE;
 
@@ -114,158 +107,236 @@ static SCM date_callback_id = SCM_UNDEFINED;
 static SCM account_separator_callback_id = SCM_UNDEFINED;
 static SCM register_colors_callback_id = SCM_UNDEFINED;
 static SCM register_borders_callback_id = SCM_UNDEFINED;
-static SCM reverse_balance_callback_id = SCM_UNDEFINED;
 static SCM auto_raise_callback_id = SCM_UNDEFINED;
 static SCM negative_color_callback_id = SCM_UNDEFINED;
 static SCM auto_decimal_callback_id = SCM_UNDEFINED;
 static SCM auto_decimal_places_callback_id = SCM_UNDEFINED;
+static SCM log_retention_days_callback_id = SCM_UNDEFINED;
+static SCM compression_callback_id = SCM_UNDEFINED;
 static SCM register_font_callback_id = SCM_UNDEFINED;
 static SCM register_hint_font_callback_id = SCM_UNDEFINED;
 
 
-/* ============================================================== */
-
 gboolean
 gnucash_ui_is_running(void)
 {
   return gnome_is_running;
 }
 
-/* ============================================================== */
-
 gboolean 
 gnucash_ui_is_terminating(void)
 {
   return gnome_is_terminating;
 }
 
-/* ============================================================== */
-
-GNCMainInfo *
-gnc_ui_get_data(void)
+static void
+gnc_global_options_help_cb (GNCOptionWin *win, gpointer dat)
 {
-  return app;
+  helpWindow (NULL, NULL, HH_GLOBPREFS);
 }
 
-gncUIWidget
-gnc_ui_get_toplevel(void) {
-  /* FIXME */
-  return gnc_main_window_get_toplevel(app);
+static gboolean
+gnc_html_register_url_cb (const char *location, const char *label,
+                          gboolean new_window, GNCURLResult *result)
+{
+  GNCSplitReg * gsr   = NULL;
+  Split       * split = NULL;
+  Account     * account;
+  Transaction * trans;
+  GList       * node;
+
+  g_return_val_if_fail (location != NULL, FALSE);
+  g_return_val_if_fail (result != NULL, FALSE);
+
+  result->load_to_stream = FALSE;
+
+  /* href="gnc-register:account=My Bank Account" */
+  if (strncmp("account=", location, 8) == 0)
+  {
+    account = xaccGetAccountFromFullName (gnc_get_current_group (),
+                                          location + 8, 
+                                          gnc_get_account_separator ());
+    gsr = regWindowSimple (account);
+    gnc_split_reg_raise( gsr );
+  }
+  /* href="gnc-register:guid=12345678901234567890123456789012" */
+  else if (strncmp ("guid=", location, 5) == 0)
+  {
+    GUID guid;
+    GNCIdType id_type;
+
+    if (!string_to_guid (location + 5, &guid))
+    {
+      result->error_message = g_strdup_printf (_("Bad URL: %s"), location);
+      return FALSE;
+    }
+
+    id_type = xaccGUIDType (&guid, gnc_get_current_book ());
+    if (id_type == GNC_ID_NONE || !safe_strcmp (id_type, GNC_ID_NULL))
+    {
+        result->error_message = g_strdup_printf (_("No such entity: %s"),
+                                                 location);
+        return FALSE;
+    }
+    else if (!safe_strcmp (id_type, GNC_ID_ACCOUNT))
+    {
+        account = xaccAccountLookup (&guid, gnc_get_current_book ());
+        gsr = regWindowSimple( account );
+    }
+    else if (!safe_strcmp (id_type, GNC_ID_TRANS))
+    {
+        trans = xaccTransLookup (&guid, gnc_get_current_book ());
+        split = NULL;
+
+        for (node = xaccTransGetSplitList (trans); node; node = node->next)
+        {
+          split = node->data;
+          if (xaccSplitGetAccount (split))
+            break;
+        }
+
+        if (!split)
+        {
+          result->error_message =
+            g_strdup_printf (_("Transaction with no Accounts: %s"), location);
+          return FALSE;
+        }
+
+        gsr = regWindowSimple( xaccSplitGetAccount(split) );
+    }
+    else if (!safe_strcmp (id_type, GNC_ID_SPLIT))
+    {
+        split = xaccSplitLookup (&guid, gnc_get_current_book ());
+        if (!split)
+        {
+          result->error_message = g_strdup_printf (_("No such split: %s"),
+                                                   location);
+          return FALSE;
+        }
+
+        gsr = regWindowSimple( xaccSplitGetAccount(split) );
+    }
+    else
+    {
+        result->error_message =
+          g_strdup_printf (_("Unsupported entity type: %s"), location);
+        return FALSE;
+    }
+
+    gnc_split_reg_raise(gsr);
+    if (split)
+      gnc_split_reg_jump_to_split( gsr, split );
+  }
+  else
+  {
+    result->error_message = g_strdup_printf (_("Badly formed URL %s"),
+                                             location);
+    return FALSE;
+  }
+
+  return TRUE;
 }
 
-gboolean
-gnc_ui_can_cancel_save (void)
+static gboolean
+gnc_html_price_url_cb (const char *location, const char *label,
+		       gboolean new_window, GNCURLResult *result)
 {
-  return gnc_main_window_can_cancel_save (app);
-}
+  g_return_val_if_fail (location != NULL, FALSE);
+  g_return_val_if_fail (result != NULL, FALSE);
 
+  result->load_to_stream = FALSE;
 
-static const char* gnc_scheme_remaining_var = "gnc:*command-line-remaining*";
+  /* href="gnc-register:guid=12345678901234567890123456789012" */
+  if (strncmp ("guid=", location, 5) == 0)
+  {
+      GUID guid;
+      GNCIdType id_type;
 
-static char**
-gnc_get_remaining_argv(int prelen, const char **prependargv)
-{
-    SCM rem = gh_eval_str (gnc_scheme_remaining_var);
-    return gnc_scheme_list_to_nulltermcharpp (prelen, prependargv, rem);
+      if (!string_to_guid (location + 5, &guid))
+      {
+	  result->error_message = g_strdup_printf (_("Bad URL: %s"), location);
+	  return FALSE;
+      }
+
+      id_type = xaccGUIDType (&guid, gnc_get_current_book ());
+      if (id_type == GNC_ID_NONE || safe_strcmp (id_type, GNC_ID_PRICE))
+      {
+          result->error_message =
+	    g_strdup_printf (_("Unsupported entity type: %s"), location);
+	  return FALSE;
+      }
+      if (!gnc_price_edit_by_guid (NULL, &guid)) {
+          result->error_message = g_strdup_printf (_("No such entity: %s"),
+						 location);
+	  return FALSE;
+      }
+  }
+  else
+  {
+      result->error_message = g_strdup_printf (_("Badly formed URL %s"),
+					       location);
+      return FALSE;
+  }
+
+  return TRUE;
 }
 
 static void
-gnc_set_remaining_argv(int len, const char **rest)
+gnc_commodity_help_cb (void)
 {
-    SCM toput = gnc_argvarr_to_scheme_list(len, rest);
-    gh_define(gnc_scheme_remaining_var, toput);
+  helpWindow (NULL, _("Help"), HH_COMMODITY);
 }
 
-             
 /* ============================================================== */
 
-/* These gnucash_ui_init and gnucash_ui functions are just hacks to get
-   the guile stuff up and running.  Expect a more formal definition of
-   what they should do soon, and expect that the open/select functions
-   will be merged with the code in FMB_OPEN in MainWindow.c */
-
-static const char *default_argv[] = {"gnucash", 0};
-
-static const struct poptOption nullPoptTable[] = {
-  { NULL, 0, 0, NULL, 0 }
-};
-
-int
-gnucash_ui_init(void)
+SCM
+gnc_gui_init_splash (SCM command_line)
 {
-  int restargc;
-  char **restargv;
-  char **restargv2;
-  poptContext returnedPoptContext;
+  SCM ret = command_line;
 
-#ifdef GTKHTML_HAVE_GCONF
-  GError *gerror;
-#endif
+  ENTER (" ");
 
-  ENTER ("\n");
-
-  /* We're going to have to have other ways to handle X and GUI
-     specific args... */
-  if (!gnome_is_initialized)
+  if (!splash_is_initialized)
   {
-    restargv = gnc_get_remaining_argv(1, default_argv);
-    if(restargv == NULL)
-    {
-      restargv = g_new(char*, 2);
-      restargv[0] = g_strdup(default_argv[0]);
-      restargv[1] = NULL;
-    }
+    splash_is_initialized = TRUE;
 
-    restargc = argv_length(restargv);
- 
-    gnome_init_with_popt_table("GnuCash", VERSION, restargc, restargv,
-                               nullPoptTable, 0, &returnedPoptContext);
-    gnome_is_initialized = TRUE;
-
-    restargv2 = (char**)poptGetArgs(returnedPoptContext);
-    gnc_set_remaining_argv(argv_length(restargv2), (const char**)restargv2);
-
-#ifdef GTKHTML_HAVE_GCONF
-    if( !gconf_init(restargc, restargv, &gerror) )
-        g_error_free(gerror);
-    gerror = NULL;
-#endif
-
-    /* this must come after using the poptGetArgs return value */
-    poptFreeContext (returnedPoptContext);
-    gnc_free_argv (restargv);
-
-    gnc_component_manager_init ();
-
-    /* initialization required for gtkhtml */
-    gdk_rgb_init ();    
-    gtk_widget_set_default_colormap (gdk_rgb_get_cmap ());
-    gtk_widget_set_default_visual (gdk_rgb_get_visual ());
-    
-    /* load default HTML action handlers */ 
-    gnc_network_init();
-
-#ifdef USE_GUPPI    
-    /* initialize guppi handling in gnc-html */
-    gnc_html_guppi_init();
-#endif
+    ret = gnc_gnome_init ("gnucash", "GnuCash", VERSION, command_line);
 
     /* put up splash screen */
     gnc_show_splash_screen ();
-    
-    /* make sure splash is up */
-    while (gtk_events_pending ())
-      gtk_main_iteration ();
+  }
+
+  LEAVE (" ");
+
+  return ret;
+}
+
+SCM
+gnc_gui_init (SCM command_line)
+{
+  SCM ret = command_line;
+
+  ENTER (" ");
+
+  if (!gnome_is_initialized)
+  {
+    /* Make sure the splash (and hense gnome) was initialized */
+    if (!splash_is_initialized)
+      ret = gnc_gui_init_splash (ret);
+
+    gnome_is_initialized = TRUE;
+
+    /* load default HTML action handlers */ 
+    // gnc_network_init();
 
     gnc_configure_date_format();
     date_callback_id =
       gnc_register_option_change_callback(gnc_configure_date_format_cb, NULL,
                                           "International", "Date Format");
 
-    gnc_configure_account_separator();
     account_separator_callback_id = 
       gnc_register_option_change_callback(gnc_configure_account_separator_cb,
-                                          NULL, "General",
+                                          NULL, "Accounts",
                                           "Account Separator");
 
     gnc_configure_register_colors();
@@ -278,16 +349,10 @@ gnucash_ui_init(void)
       gnc_register_option_change_callback(gnc_configure_register_borders_cb,
                                           NULL, "Register", NULL);
     
-    gnc_configure_reverse_balance();
-    reverse_balance_callback_id = 
-      gnc_register_option_change_callback(gnc_configure_reverse_balance_cb,
-                                          NULL, "General",
-                                          "Reversed-balance account types");
-
     gnc_configure_auto_raise();
     auto_raise_callback_id = 
       gnc_register_option_change_callback(gnc_configure_auto_raise_cb,
-                                          NULL, "Register",
+                                          NULL, "_+Advanced",
                                           "Auto-Raise Lists");
 
     gnc_configure_negative_color();
@@ -308,6 +373,18 @@ gnucash_ui_init(void)
                                           NULL, "General",
                                           "Auto Decimal Places");
 
+    gnc_configure_file_be_retention_days();
+    log_retention_days_callback_id = 
+      gnc_register_option_change_callback(gnc_configure_file_be_retention_days_cb,
+                                          NULL, "General",
+                                          "Days to retain log files");
+
+    gnc_configure_file_be_compression();
+    compression_callback_id = 
+      gnc_register_option_change_callback(gnc_configure_file_be_compression_cb,
+                                          NULL, "General",
+                                          "Use file compression");
+
     gnc_configure_register_font();
     register_font_callback_id =
       gnc_register_option_change_callback(gnc_configure_register_font_cb,
@@ -319,14 +396,27 @@ gnucash_ui_init(void)
                                           NULL, "Register",
                                           "Register hint font");
 
-    xaccRecnCellSetStringGetter(gnc_get_reconcile_str);
-
-    /* gnc_default_ui_start */
-    gnucash_style_init();
+    if (!gnucash_style_init())
+      gnc_shutdown(1);
     gnucash_color_init();
 
+    gnc_html_register_url_handler (URL_TYPE_REGISTER,
+                                   gnc_html_register_url_cb);
+
+    gnc_html_register_url_handler (URL_TYPE_PRICE,
+                                   gnc_html_price_url_cb);
+
+    gnc_ui_commodity_set_help_callback (gnc_commodity_help_cb);
+
+    gnc_file_set_can_cancel_callback (gnc_mdi_has_apps);
+    gnc_file_set_shutdown_callback (gnc_shutdown);
+
+    gnc_options_dialog_set_global_help_cb (gnc_global_options_help_cb, NULL);
+
     /* initialize gnome MDI and set up application window defaults  */
-    app = gnc_main_window_new();
+    if (!gnc_mdi_get_current ())
+      gnc_main_window_new ();
+
     /* Run the ui startup hooks. */
     {
       SCM run_danglers = gh_eval_str("gnc:hook-run-danglers");
@@ -335,42 +425,30 @@ gnucash_ui_init(void)
     }    
   }
 
-  LEAVE ("\n");
+  LEAVE (" ");
 
-  return 0;
-}
-
-static gboolean hasstarted = FALSE;
-void
-gnc_default_ui_start(void) {
-  if(!hasstarted) {
-    hasstarted = TRUE;
-    gncGetCurrentBook();
-  }
+  return ret;
 }
 
 /* ============================================================== */
 
 void
-gnc_ui_shutdown (void)
+gnc_gui_shutdown (void)
 {
   if (gnome_is_running && !gnome_is_terminating)
   {
     gnome_is_terminating = TRUE;
-    /*    gnc_main_window_save(app); */
-    gnc_main_window_destroy(app);
-    app = NULL;
+
     gtk_main_quit();
-#ifdef USE_GUPPI    
-    gnc_html_guppi_shutdown();
-#endif
+
+    gnc_gnome_shutdown ();
   }
 }
 
 /* ============================================================== */
 
 void
-gnc_ui_destroy (void)
+gnc_gui_destroy (void)
 {
   if (!gnome_is_initialized)
     return;
@@ -379,61 +457,40 @@ gnc_ui_destroy (void)
   gnc_unregister_option_change_callback_id(account_separator_callback_id);
   gnc_unregister_option_change_callback_id(register_colors_callback_id);
   gnc_unregister_option_change_callback_id(register_borders_callback_id);
-  gnc_unregister_option_change_callback_id(reverse_balance_callback_id);
   gnc_unregister_option_change_callback_id(auto_raise_callback_id);
   gnc_unregister_option_change_callback_id(negative_color_callback_id);
   gnc_unregister_option_change_callback_id(register_font_callback_id);
   gnc_unregister_option_change_callback_id(register_hint_font_callback_id);
 
-  if (app != NULL)
-  {
-    gnc_main_window_destroy(app);
-    app = NULL;
-  }
+  gnc_mdi_destroy (gnc_mdi_get_current ());
 
   gnc_extensions_shutdown ();
-
-  gnc_component_manager_shutdown ();
 }
 
 /* ============================================================== */
 
-int
-gnc_ui_show_main_window(void)
-{
-  /* Initialize gnome */
-  gnucash_ui_init();
-  gnc_default_ui_start();
-  
-  /* Get the main window on screen. */
-  while (gtk_events_pending())
-    gtk_main_iteration();
-
-  return 0;
-}
-
 static gboolean
 gnc_ui_check_events (gpointer not_used)
 {
-  GNCBook *book;
+  GNCSession *session;
   gboolean force;
 
   if (gtk_main_level() != 1)
     return TRUE;
 
-  book = gncGetCurrentBook ();
-  if (!book)
+  session = gnc_get_current_session ();
+  if (!session)
     return TRUE;
 
   if (gnc_gui_refresh_suspended ())
     return TRUE;
 
-  if (!gnc_book_events_pending (book))
+  if (!gnc_session_events_pending (session))
     return TRUE;
 
   gnc_suspend_gui_refresh ();
 
-  force = gnc_book_process_events (book);
+  force = gnc_session_process_events (session);
 
   gnc_resume_gui_refresh ();
 
@@ -488,48 +545,6 @@ gnc_ui_start_event_loop (void)
   return 0;
 }
 
-
-int
-gnc_ui_main(void)
-{
-    gnc_ui_show_main_window();
-
-    return gnc_ui_start_event_loop();
-}
-
-/* ============================================================== */
-
-gboolean
-gnucash_ui_open_file(const char name[])
-{
-  return gncFileOpenFile(name);
-}
-
-/* ============================================================== */
-
-int
-gnucash_ui_select_file(void)
-{
-  gncFileOpen();
-  return 1;
-}
-
-/* ============================================================== */
-
-const char *
-gnc_register_default_font(void)
-{
-  return gnucash_style_get_default_register_font_name();
-}
-
-/* ============================================================== */
-
-const char *
-gnc_register_default_hint_font(void)
-{
-  return gnucash_style_get_default_register_hint_font_name();
-}
-
 /* ============================================================== */
 
 /* gnc_configure_date_format_cb
@@ -558,7 +573,7 @@ gnc_configure_date_format (void)
 {
   char *format_code = gnc_lookup_multichoice_option("International", 
                                                     "Date Format",
-                                                    "us");
+                                                    "locale");
 
   DateFormat df;
 
@@ -609,25 +624,9 @@ gnc_configure_date_format (void)
 static void 
 gnc_configure_account_separator_cb (gpointer data)
 {
-  gnc_configure_account_separator ();
   gnc_gui_refresh_all ();
 }
 
-/* gnc_configure_account_separator
- *    sets the account separator to the
- *    current value on the scheme side
- *
- * Args: Nothing
- * Returns: Nothing
- */
-static void
-gnc_configure_account_separator (void)
-{
-  char separator = gnc_get_account_separator ();
-
-  xaccSRSetAccountSeparator (separator);
-}
-
 /* gnc_configure_register_colors_cb
  *    Callback called when options change - sets
  *    register colors to their guile values
@@ -693,7 +692,7 @@ gnc_configure_register_colors (void)
                               "Double mode colors alternate with transactions",
                               FALSE);
 
-  xaccSetSplitRegisterColors (reg_colors);
+  gnc_split_register_set_colors (reg_colors);
 }
 
 
@@ -723,12 +722,12 @@ gnc_configure_register_borders (void)
   gboolean use_vertical_lines;
   gboolean use_horizontal_lines;
 
-  use_vertical_lines = gnc_lookup_boolean_option("Register",
+  use_vertical_lines = gnc_lookup_boolean_option("_+Advanced",
                                                  "Show Vertical Borders",
                                                  FALSE);
 
   
-  use_horizontal_lines = gnc_lookup_boolean_option("Register",
+  use_horizontal_lines = gnc_lookup_boolean_option("_+Advanced",
                                                    "Show Horizontal Borders",
                                                    FALSE);
 
@@ -760,9 +759,9 @@ gnc_configure_auto_raise (void)
 {
   gboolean auto_pop;
 
-  auto_pop = gnc_lookup_boolean_option("Register", "Auto-Raise Lists", TRUE);
+  auto_pop = gnc_lookup_boolean_option("_+Advanced", "Auto-Raise Lists", TRUE);
 
-  xaccComboCellSetAutoPop(auto_pop);
+  gnc_combo_cell_set_autopop (auto_pop);
 }
 
 /* gnc_configure_negative_color_cb
@@ -795,96 +794,9 @@ gnc_configure_negative_color(void)
                                       "Display negative amounts in red",
                                       TRUE);
 
-  xaccSetSplitRegisterColorizeNegative (use_red);
+  gnc_split_register_colorize_negative (use_red);
 }
 
-/* gnc_configure_reverse_balance_cb
- *    Callback called when options change - sets
- *    reverse balance info for the callback
- *
- * Args: Nothing
- * Returns: Nothing
- */
-static void
-gnc_configure_reverse_balance_cb (gpointer not_used)
-{
-  gnc_configure_reverse_balance ();
-  gnc_gui_refresh_all ();
-}
-
-static gboolean reverse_type[NUM_ACCOUNT_TYPES];
-
-gboolean
-gnc_reverse_balance_type(GNCAccountType type)
-{
-  if ((type < 0) || (type >= NUM_ACCOUNT_TYPES))
-    return FALSE;
-
-  return reverse_type[type];
-}
-
-gboolean
-gnc_reverse_balance(Account *account)
-{
-  int type;
-
-  if (account == NULL)
-    return FALSE;
-
-  type = xaccAccountGetType(account);
-  if ((type < 0) || (type >= NUM_ACCOUNT_TYPES))
-    return FALSE;
-
-  return reverse_type[type];
-}
-
-/* gnc_configure_reverse_balance
- *    sets reverse balance info for the callback
- *
- * Args: Nothing
- * Returns: Nothing
- */
-static void
-gnc_configure_reverse_balance(void)
-{
-  gchar *choice;
-  gint i;
-
-  xaccSRSetReverseBalanceCallback(gnc_reverse_balance);
-
-  for (i = 0; i < NUM_ACCOUNT_TYPES; i++)
-    reverse_type[i] = FALSE;
-
-  choice = gnc_lookup_multichoice_option("General",
-                                         "Reversed-balance account types",
-                                         "credit");
-
-  if (safe_strcmp(choice, "income-expense") == 0)
-  {
-    reverse_type[INCOME]  = TRUE;
-    reverse_type[EXPENSE] = TRUE;
-  }
-  else if (safe_strcmp(choice, "credit") == 0)
-  {
-    reverse_type[LIABILITY] = TRUE;
-    reverse_type[EQUITY]    = TRUE;
-    reverse_type[INCOME]    = TRUE;
-    reverse_type[CREDIT]    = TRUE;
-  }
-  else if (safe_strcmp(choice, "none") == 0)
-  {
-  }
-  else
-  {
-    PERR("bad value\n");
-
-    reverse_type[INCOME]  = TRUE;
-    reverse_type[EXPENSE] = TRUE;
-  }
-
-  if (choice != NULL)
-    free(choice);
-}
 
 /* gnc_configure_auto_decimal_cb
  *     Callback called when options change -
@@ -945,6 +857,59 @@ gnc_configure_auto_decimal_places (void)
 }
 
 
+/* gnc_configure_file_be_retention_days_cb
+ *     Callback called when options change -
+ *     sets days retained for the file backend.
+ * 
+ *  Args: Nothing
+ *  Returns: Nothing
+ */
+static void
+gnc_configure_file_be_retention_days_cb (gpointer not_used)
+{
+  gnc_configure_file_be_retention_days ();
+}
+
+/* gnc_configure_file_be_retention_days
+ *     Pass the global value for the number of days to retain files to the file backend.
+ * 
+ * Args: Nothing
+ * Returns: Nothing
+ */
+static void
+gnc_configure_file_be_retention_days (void)
+{
+  gnc_file_be_set_retention_days
+    (gnc_lookup_number_option("General",
+			      "Days to retain log files", 0));
+}
+
+/* gnc_configure_file_be_retention_days_cb
+ *     Callback called when options change -
+ *     sets days retained for the file backend.
+ * 
+ *  Args: Nothing
+ *  Returns: Nothing
+ */
+static void
+gnc_configure_file_be_compression_cb (gpointer not_used)
+{
+  gnc_configure_file_be_compression ();
+}
+
+/* gnc_configure_file_be_retention_days
+ *     Pass the global value for the number of days to retain files to the file backend.
+ * 
+ * Args: Nothing
+ * Returns: Nothing
+ */
+static void
+gnc_configure_file_be_compression (void)
+{
+  gnc_file_be_set_compression
+    (gnc_lookup_boolean_option("General", "Use file compression", FALSE));
+}
+
 /* gnc_configure_register_font_cb
  *     Callback called when options change -
  *     sets register font
