@@ -21,17 +21,27 @@
  *                                                                  *
 \********************************************************************/
 
-#ifndef __WINDOW_REGISTER_H__
-#define __WINDOW_REGISTER_H__
+#ifndef WINDOW_REGISTER_H
+#define WINDOW_REGISTER_H
 
-#include "RegWindow.h"
-#include "MultiLedger.h"
+#include "gnc-ledger-display.h"
+#include "gnc-split-reg.h"
 
-RegWindow * regWindowLedger(xaccLedgerDisplay *ledger);
+/** STRUCTS *********************************************************/
+typedef struct _RegWindow RegWindow;
+/* Getters */
+GtkWidget *gnc_RegWindow_window (RegWindow *data);
+GNCLedgerDisplay *gnc_RegWindow_ledger (RegWindow *data);
+
+/** PROTOTYPES ******************************************************/
+GNCSplitReg* regWindowSimple(Account *account);
+GNCSplitReg* regWindowAccGroup(Account *account_group);
+
+RegWindow* regWindowLedger(GNCLedgerDisplay *ledger);
+
+gpointer gnc_RegWindow_get_pcd (RegWindow *data);
+void     gnc_RegWindow_set_pcd (RegWindow *data, gpointer);
 
 void gnc_register_raise(RegWindow *regData);
-void gnc_register_jump_to_blank(RegWindow *regData);
-void gnc_register_jump_to_split(RegWindow *regData, Split *split);
-void gnc_register_jump_to_split_amount(RegWindow *regData, Split *split);
 
 #endif
