@@ -25,8 +25,7 @@
 
 #include <gnome.h>
 #include <errno.h>
-#include <openhbci.h>
-#include <openhbci/error.h>
+#include <gwenhywfar/directory.h>
 
 #include "gnc-ui.h"
 #include "gnc-hbci-kvp.h"
@@ -35,151 +34,120 @@
 #include "global-options.h"
 
 #include "hbci-interaction.h"
+#include <aqbanking/version.h>
 
 /* static short module = MOD_IMPORT; */
 
-HBCI_API *
-gnc_hbci_api_new (const char *filename, gboolean allowNewFile,
-		  GtkWidget *parent, GNCInteractor **inter)
-{
-  HBCI_API *api = NULL;
-  HBCI_Error *err = NULL;
-  char *errstring;
-  
-  if (!filename)
-      return NULL;
-  if (!allowNewFile && 
-      (!g_file_test (filename, G_FILE_TEST_ISFILE | G_FILE_TEST_ISLINK))) 
-    {
-      /* ENOENT is "No such file or directory" */
-      gchar *errstring = g_strdup_printf ("%s: %s", filename, strerror (ENOENT));
-      gnc_warning_dialog_parented 
-	(parent,
-	 /* Translators: Strings from this file are really only needed
-	  * inside Germany (HBCI is not supported anywhere else). You
-	  * may safely ignore strings from the import-export/hbci
-	  * subdirectory in other countries. */
-	 _("Error while loading OpenHBCI config file:\n  %s\n"), errstring);
-      g_free (errstring);
-      return NULL;
-    }
-  
-
-  api = HBCI_API_new (FALSE, TRUE);
-  
-  err = HBCI_API_loadEnvironment (api, filename);
-  if (!HBCI_Error_isOk (err) && !allowNewFile) {
-    errstring = HBCI_Error_errorString (err);
-    HBCI_Error_delete (err);
-    gnc_warning_dialog_parented 
-	(parent,
-	 /* Translators: Strings from this file are really only needed
-	  * inside Germany (HBCI is not supported anywhere else). You
-	  * may safely ignore strings from the import-export/hbci
-	  * subdirectory in other countries. */
-	 _("Error while loading OpenHBCI config file:\n  %s\n"), errstring);
-    free (errstring);
-    HBCI_API_delete (api);
-    return NULL;
-  }
-  HBCI_Error_delete (err);
-
-  if (inter)
-    *inter = gnc_hbci_api_interactors (api, parent);
-  else
-    gnc_hbci_api_interactors (api, parent);
-
-  return api;
-}
-
-static HBCI_API *gnc_hbci_api = NULL;
-static char *gnc_hbci_configfile = NULL;
+/* Globale variables for AB_BANKING caching. */
+static AB_BANKING *gnc_AB_BANKING = NULL;
+static int gnc_AB_BANKING_refcnt = 0;
 static GNCInteractor *gnc_hbci_inter = NULL;
 
-HBCI_API * gnc_hbci_api_new_currentbook (GtkWidget *parent, 
-					 GNCInteractor **inter)
+
+AB_BANKING * gnc_AB_BANKING_new_currentbook (GtkWidget *parent, 
+					     GNCInteractor **inter)
 {
-  if (gnc_hbci_api == NULL) {
+  if (gnc_AB_BANKING == NULL) {
     /* No API cached -- create new one. */
-    gnc_hbci_configfile = 
-      g_strdup (gnc_hbci_get_book_configfile (gnc_get_current_book ()));
-    gnc_hbci_api = gnc_hbci_api_new (gnc_hbci_configfile, 
-				     FALSE, parent, inter);
-    gnc_hbci_inter = *inter;
-    return gnc_hbci_api;
-  } else if ((gnc_hbci_configfile != NULL) && 
-	     (strcmp(gnc_hbci_configfile, 
-		     gnc_hbci_get_book_configfile (gnc_get_current_book ()))
-	     != 0)) {
-    /* Wrong API cached -- delete old and create new. */
-    gnc_hbci_api_delete (gnc_hbci_api);
-    fprintf(stderr,
-	    "gnc_hbci_api_new_currentbook: Wrong HBCI_API cached; creating new one.\n");
-    return gnc_hbci_api_new_currentbook (parent, inter);
-  } else {
-    /* Correct API cached. */
-    *inter = gnc_hbci_inter;
-    GNCInteractor_reparent (*inter, parent);
-    return gnc_hbci_api;
-  }
-}
-
-void gnc_hbci_api_delete (HBCI_API *api)
-{
-  if (api == gnc_hbci_api) {
-    gnc_hbci_api = NULL;
-    gnc_hbci_inter = NULL;
-    g_free (gnc_hbci_configfile);
-    gnc_hbci_configfile = NULL;
-  }
-  HBCI_API_delete (api);
-}
-
-
-HBCI_Error * gnc_hbci_api_save (const HBCI_API *api)
-{
-  const char *file = gnc_hbci_get_book_configfile (gnc_get_current_book ());
-  if ((file == NULL) || (strlen (file) == 0)) 
-    return HBCI_Error_new ("gnc_hbci_api_save", ERROR_LEVEL_NORMAL, 0, 
-			   ERROR_ADVISE_ABORT, 
-			   "No filename for config file.", "");
+    AB_BANKING *api = NULL;
   
-  return HBCI_API_saveEnvironment (api, file);
+    api = AB_Banking_new ("gnucash", 0);
+    g_assert(api);
+    {
+      int r = AB_Banking_Init(api);
+      if (r != 0)
+	printf("gnc_AB_BANKING_new: Warning: Error %d on AB_Banking_init\n", r);
+    }
+    
+    gnc_hbci_inter = gnc_AB_BANKING_interactors (api, parent);
+    gnc_AB_BANKING = api;
+    
+    if (inter)
+      *inter = gnc_hbci_inter;
+
+    gnc_AB_BANKING_refcnt = 1;
+    return gnc_AB_BANKING;
+  } else {
+    /* API cached. */
+
+    /* Init the API again. */
+    if (gnc_AB_BANKING_refcnt == 0)
+      AB_Banking_Init(gnc_AB_BANKING);
+
+    if (inter) {
+      *inter = gnc_hbci_inter;
+      GNCInteractor_reparent (*inter, parent);
+    }
+    
+    gnc_AB_BANKING_refcnt++;
+    return gnc_AB_BANKING;
+  }
+}
+
+void gnc_AB_BANKING_delete (AB_BANKING *api)
+{
+  if (api == 0)
+    api = gnc_AB_BANKING;
+
+  if (api) {
+    if (api == gnc_AB_BANKING) {
+      gnc_AB_BANKING = NULL;
+      gnc_hbci_inter = NULL;
+      if (gnc_AB_BANKING_refcnt > 0)
+	AB_Banking_Fini(api);
+    }
+
+    AB_Banking_free(api);
+  }
+}
+
+
+int gnc_AB_BANKING_fini (AB_BANKING *api) 
+{
+  if (api == gnc_AB_BANKING) {
+    gnc_AB_BANKING_refcnt--;
+    if (gnc_AB_BANKING_refcnt == 0)
+      return AB_Banking_Fini(api);
+  }
+  else
+    return AB_Banking_Fini(api);
+  return 0;
 }
 
 
 
-const HBCI_Account *
-gnc_hbci_get_hbci_acc (const HBCI_API *api, Account *gnc_acc) 
+
+AB_ACCOUNT *
+gnc_hbci_get_hbci_acc (const AB_BANKING *api, Account *gnc_acc) 
 {
+  int account_uid = 0;
+  AB_ACCOUNT *hbci_acc = NULL;
   const char *bankcode = NULL, *accountid = NULL;
-  int countrycode = 0;
-  const HBCI_Bank *bank = NULL;
-  const HBCI_Account *hbci_acc = NULL;
 
   bankcode = gnc_hbci_get_account_bankcode (gnc_acc);
-  countrycode = gnc_hbci_get_account_countrycode (gnc_acc);
-  if (bankcode && (strlen(bankcode)>0) && (countrycode > 0)) {
-    /*printf("gnc_acc %s has blz %s and ccode %d\n",
+  accountid = gnc_hbci_get_account_accountid (gnc_acc);
+  account_uid = gnc_hbci_get_account_uid (gnc_acc);
+  if (account_uid > 0) {
+    /*printf("gnc_hbci_get_hbci_acc: gnc_acc %s has blz %s and ccode %d\n",
       xaccAccountGetName (gnc_acc), bankcode, countrycode);*/
-    bank = HBCI_API_findBank (api, countrycode, bankcode);
-    if (bank) {
-      accountid = gnc_hbci_get_account_accountid (gnc_acc);
-      if (accountid && (strlen(accountid)>0)) {
-	hbci_acc = HBCI_Bank_findAccount (bank, accountid);
-	if (hbci_acc) {
-	  /*printf("can connect gnc_acc %s to hbci_acc %s\n",
-	    xaccAccountGetName (gnc_acc), 
-	    HBCI_Account_accountId (hbci_acc));*/
-	  return hbci_acc;
-	} /* hbci_acc */
-      } /* accountid */
-    } /* bank */
-  } /* bankcode */
+    hbci_acc = AB_Banking_GetAccount(api, account_uid);
+
+    if (!hbci_acc && bankcode && (strlen(bankcode)>0) &&
+	accountid && (strlen(accountid) > 0)) {
+      printf("gnc_hbci_get_hbci_acc: No AB_ACCOUNT found for UID %d, trying bank code\n", account_uid);
+      hbci_acc = AB_Banking_GetAccountByCodeAndNumber(api, bankcode, accountid);
+    }
+    /*printf("gnc_hbci_get_hbci_acc: return HBCI_Account %p\n", hbci_acc);*/
+    return hbci_acc;
+  } else if (bankcode && (strlen(bankcode)>0) && accountid && (strlen(accountid) > 0)) {
+    hbci_acc = AB_Banking_GetAccountByCodeAndNumber(api, bankcode, accountid);
+    return hbci_acc;
+  }
+
   return NULL;
 }
 
-
+#if 0
 static void *
 print_list_int_cb (int value, void *user_data)
 {
@@ -212,52 +180,29 @@ get_resultcode_error (const list_int *list)
   cause = (int) list_int_foreach (list, &get_resultcode_error_cb, &tmp_result);
   return MAX(tmp_result, cause);
 }
-
+#endif
 int
-gnc_hbci_debug_outboxjob (HBCI_OutboxJob *job, gboolean verbose)
+gnc_hbci_debug_outboxjob (AB_JOB *job, gboolean verbose)
 {
-  list_int *list;
-  const char *msg;
+/*   list_int *list; */
+/*   const char *msg; */
   int cause = 0;
   
   g_assert (job);
-/*   if (HBCI_OutboxJob_status (job) != HBCI_JOB_STATUS_DONE) */
+/*   if (AB_JOB_status (job) != HBCI_JOB_STATUS_DONE) */
 /*     return; */
-/*   if (HBCI_OutboxJob_result (job) != HBCI_JOB_RESULT_FAILED) */
+/*   if (AB_JOB_result (job) != HBCI_JOB_RESULT_FAILED) */
 /*     return; */
 
   if (verbose) {
-    printf("OutboxJob status: ");
-    switch(HBCI_OutboxJob_status (job)) {
-    case HBCI_JOB_STATUS_TODO:
-      printf("todo");
-      break;
-    case HBCI_JOB_STATUS_DONE:
-      printf("done");
-      break;
-    default:
-    case HBCI_JOB_STATUS_NONE:
-      printf("none");
-      break;
-    }
+    printf("OutboxJob status: %s", AB_Job_Status2Char(AB_Job_GetStatus(job)));
 
-    printf(", result: ");
-    switch(HBCI_OutboxJob_result (job)) {
-    case HBCI_JOB_RESULT_SUCCESS:
-      printf("success");
-      break;
-    case HBCI_JOB_RESULT_FAILED:
-      printf("failed");
-      break;
-    default:
-    case HBCI_JOB_STATUS_NONE:
-      printf("none");
-      break;
-    }
+    printf(", result: %s", AB_Job_GetResultText(job));
     printf("\n");
   }
-  
-  list = HBCI_OutboxJob_resultCodes (job);
+
+#if 0  
+  list = AB_JOB_resultCodes (job);
   if (list_int_size (list) > 0) {
 
     cause = get_resultcode_error (list);
@@ -289,99 +234,104 @@ gnc_hbci_debug_outboxjob (HBCI_OutboxJob *job, gboolean verbose)
       printf("OutboxJob's resultCodes list has zero length.\n");
   }
   list_int_delete (list);
+#endif
 
   return cause;
 }
 
 
 gboolean
-gnc_hbci_error_retry (GtkWidget *parent, HBCI_Error *error,
+gnc_hbci_Error_retry (GtkWidget *parent, int error, 
 		      GNCInteractor *inter)
 {
-  int code = HBCI_Error_code (error);
+  int code = error;
 
   switch (code) {
-  case HBCI_ERROR_CODE_PIN_WRONG:
+#if 0
+  case AB_ERROR_PIN_WRONG:
     GNCInteractor_erasePIN (inter);
     return gnc_verify_dialog_parented (parent,
 				       TRUE,
 				       _("The PIN you entered was wrong.\n"
 					 "Do you want to try again?"));
-  case HBCI_ERROR_CODE_PIN_WRONG_0:
+  case AB_ERROR_PIN_WRONG_0:
     GNCInteractor_erasePIN (inter);
     return gnc_verify_dialog_parented (parent,
 				       TRUE,
 				       _("The PIN you entered was wrong.\n"
 					 "ATTENTION: You have zero further wrong retries left!\n"
 					 "Do you want to try again?"));
-  case HBCI_ERROR_CODE_PIN_WRONG_1:
+  case AB_ERROR_PIN_WRONG_1:
     GNCInteractor_erasePIN (inter);
     return gnc_verify_dialog_parented (parent,
 				       TRUE,
 				       _("The PIN you entered was wrong.\n"
 					 "You have one further wrong retry left.\n"
 					 "Do you want to try again?"));
-  case HBCI_ERROR_CODE_PIN_WRONG_2:
+  case AB_ERROR_PIN_WRONG_2:
     GNCInteractor_erasePIN (inter);
     return gnc_verify_dialog_parented (parent,
 				       TRUE,
 				       _("The PIN you entered was wrong.\n"
 					 "You have two further wrong retries left.\n"
 					 "Do you want to try again?"));
-  case HBCI_ERROR_CODE_PIN_ABORTED:
-    /*     printf("gnc_hbci_error_feedback: PIN dialog was aborted.\n"); */
+  case AB_ERROR_PIN_ABORTED:
+    /*     printf("gnc_hbci_Error_feedback: PIN dialog was aborted.\n"); */
     return FALSE;
-  case HBCI_ERROR_CODE_PIN_TOO_SHORT:
+  case AB_ERROR_PIN_TOO_SHORT:
     GNCInteractor_erasePIN (inter);
     return gnc_verify_dialog_parented (parent,
 				       TRUE,
 				       _("The PIN you entered was too short.\n"
 					 "Do you want to try again?"));
-  case HBCI_ERROR_CODE_CARD_DESTROYED:
+  case AB_ERROR_CARD_DESTROYED:
     GNCInteractor_hide (inter);
     gnc_error_dialog_parented
-      (GTK_WINDOW (parent),
+      (parent,
        _("Unfortunately you entered a wrong PIN for too many times.\n"
 	 "Your chip card is therefore destroyed. Aborting."));
     return FALSE;
-  case HBCI_ERROR_CODE_FILE_NOT_FOUND:
-    /*     printf("gnc_hbci_error_feedback: File not found error.\n"); */
+  case AB_ERROR_FILE_NOT_FOUND:
+    /*     printf("gnc_hbci_Error_feedback: File not found error.\n"); */
     return FALSE;
-  case HBCI_ERROR_CODE_NO_CARD:
+  case AB_ERROR_NO_CARD:
     return gnc_verify_dialog_parented (parent,
 				       TRUE,
 				       _("No chip card has been found in the chip card reader.\n"
 					 "Do you want to try again?"));
-  case HBCI_ERROR_CODE_JOB_NOT_SUPPORTED:
+  case AB_ERROR_JOB_NOT_SUPPORTED:
     GNCInteractor_hide (inter);
     gnc_error_dialog_parented 
-      (GTK_WINDOW (parent),
+      (parent,
        _("Unfortunately this HBCI job is not supported \n"
 	 "by your bank or for your account. Aborting."));
     return FALSE;
-  case HBCI_ERROR_CODE_SOCKET_NO_CONNECT:
+#endif
+  case AB_ERROR_NETWORK:
     GNCInteractor_hide (inter);
-    gnc_error_dialog_parented 
+    gnc_error_dialog_parented
       (GTK_WINDOW (parent),
        _("The server of your bank refused the HBCI connection.\n"
 	 "Please try again later. Aborting."));
     return FALSE;
-  case HBCI_ERROR_CODE_MEDIUM:
+#if 0
+  case AB_ERROR_MEDIUM:
     gnc_error_dialog_parented 
-      (GTK_WINDOW (parent),
+      (parent,
        _("There was an error when loading the plugin for your security medium \n"
 	 "(see log window). Probably the versions of your currently installed \n"
 	 "OpenHBCI library and of the plugin do not match. In that case you need \n"
 	 "to recompile and reinstall the plugin again. Aborting now."));
     GNCInteractor_hide (inter);
     return FALSE;
-  case HBCI_ERROR_CODE_BAD_MEDIUM:
-    gnc_error_dialog_parented 
-      (GTK_WINDOW (parent),
+  case AB_ERROR_BAD_MEDIUM:
+    gnc_error_dialog_parented
+      (parent,
        _("Your security medium is not supported. No appropriate plugin \n"
 	 "has been found for that medium. Aborting."));
     GNCInteractor_hide (inter);
     return FALSE;
+#endif
       
   default:
     return FALSE;
@@ -389,39 +339,112 @@ gnc_hbci_error_retry (GtkWidget *parent, HBCI_Error *error,
   
 }
 
-gboolean
-gnc_hbci_api_execute (GtkWidget *parent, HBCI_API *api,
-		      HBCI_OutboxJob *job, GNCInteractor *inter)
+#if 0
+/* Prints all results that can be found in the outbox into the interactor */
+static void gnc_hbci_printresult(HBCI_Outbox *outbox, GNCInteractor *inter)
 {
-  HBCI_Error *err;
+  /* Got no sysid. */
+  GWEN_DB_NODE *rsp, *n;
+  g_assert(outbox);
+  if (!inter) 
+    return;
+  
+  rsp = HBCI_Outbox_response(outbox);
+  n = GWEN_DB_GetFirstGroup(rsp);
+  while(n) {
+    if (strcasecmp(GWEN_DB_GroupName(n), "msgresult")==0) {
+      GWEN_DB_NODE *r = GWEN_DB_GetFirstGroup(n);
+      while (r) {
+	if (strcasecmp(GWEN_DB_GroupName(r), "result") == 0) {
+	  gchar *logtext;
+	  int resultcode;
+	  const char *text, *elementref, *param;
+	  
+	  resultcode = GWEN_DB_GetIntValue(r, "resultcode", 0, 0);
+	  text = GWEN_DB_GetCharValue(r, "text", 0, "Response without text");
+	  elementref = GWEN_DB_GetCharValue(r, "elementref", 0, "");
+	  param = GWEN_DB_GetCharValue(r, "param", 0, "");
+
+	  if (strlen(elementref)>0 || strlen(param) > 0)
+	    logtext = g_strdup_printf("%s (%d; Elementref %s; Param %s)", text, 
+				      resultcode, elementref, param);
+	  else
+	    logtext = g_strdup_printf("%s (%d)", text, resultcode);
+	  GNCInteractor_add_log_text(inter, logtext);
+	  g_free(logtext);
+	}
+	r = GWEN_DB_GetNextGroup(r);
+      }
+    } 
+    else if (strcasecmp(GWEN_DB_GroupName(n), "segresult")==0) {
+      GWEN_DB_NODE *r = GWEN_DB_GetFirstGroup(n);
+      while (r) {
+	if (strcasecmp(GWEN_DB_GroupName(r), "result") == 0) {
+	}
+	r = GWEN_DB_GetNextGroup(r);
+      }
+    } 
+    n=GWEN_DB_GetNextGroup(n);
+  } /* while */
+
+  GWEN_DB_Group_free(rsp);
+}
+#endif
+
+static gboolean hbci_Error_isOk(int err) {
+  switch (err) {
+  case 0:
+    return TRUE;
+  default:
+    return FALSE;
+  };
+}
+
+gboolean
+gnc_AB_BANKING_execute (GtkWidget *parent, AB_BANKING *api,
+			AB_JOB *job, GNCInteractor *inter)
+{
+  int err;
   int resultcode;
 	  
   if (inter)
     GNCInteractor_show (inter);
 
   if (gnc_lookup_boolean_option("_+Advanced", 
-				"HBCI Verbose Debug Messages", FALSE))
-    HBCI_Hbci_setDebugLevel (4);
-  else
-    HBCI_Hbci_setDebugLevel (0);
+				"HBCI Verbose Debug Messages", FALSE)) {
+/*     GWEN_Logger_SetLevel(0, GWEN_LoggerLevelDebug); */
+/*     HBCI_Hbci_setDebugLevel (4); */
+  }
+/*   else */
+/*     HBCI_Hbci_setDebugLevel (0); */
 
   do {
-    if (inter)
+    if (inter) {
       GNCInteractor_show_nodelete (inter);
-    err = HBCI_API_executeQueue (api, TRUE);
-    g_assert (err);
-  } while (gnc_hbci_error_retry (parent, err, inter));
+#if (AQBANKING_VERSION_MAJOR > 0) || (AQBANKING_VERSION_MINOR > 9) || \
+  ((AQBANKING_VERSION_MINOR == 9) && \
+   ((AQBANKING_VERSION_PATCHLEVEL > 7) || \
+    ((AQBANKING_VERSION_PATCHLEVEL == 7) && (AQBANKING_VERSION_BUILD > 1))))
+      AB_Banking_SetPinCacheEnabled (api, GNCInteractor_get_cache_valid(inter));
+#endif
+    }
+
+    err = AB_Banking_ExecuteQueue (api);
+
+    /* Print result codes to interactor */
+/*     gnc_hbci_printresult(queue, inter); */
+    
+  } while (gnc_hbci_Error_retry (parent, err, inter));
   
   resultcode = gnc_hbci_debug_outboxjob (job, FALSE);
-  if (!HBCI_Error_isOk(err)) {
-    char *errstr = 
-      g_strdup_printf("gnc_hbci_api_execute: Error at executeQueue: %s",
-		      HBCI_Error_message (err));
-    printf("%s\n", errstr);
-    HBCI_Interactor_msgStateResponse (HBCI_Hbci_interactor 
-				      (HBCI_API_Hbci (api)), errstr);
-    g_free (errstr);
-    HBCI_Error_delete (err);
+  if (!hbci_Error_isOk(err)) {
+/*     char *errstr =  */
+/*       g_strdup_printf("gnc_AB_BANKING_execute: Error at executeQueue: %s", */
+/* 		      hbci_Error_message (err)); */
+/*     printf("%s\n", errstr); */
+/*     HBCI_Interactor_msgStateResponse (HBCI_Hbci_interactor  */
+/* 				      (AB_BANKING_Hbci (api)), errstr); */
+/*     g_free (errstr); */
     gnc_hbci_debug_outboxjob (job, TRUE);
     GNCInteractor_show_nodelete (inter);
     return FALSE;
@@ -429,13 +452,11 @@ gnc_hbci_api_execute (GtkWidget *parent, HBCI_API *api,
 
   GNCInteractor_set_cache_valid (inter, TRUE);
   if (resultcode <= 20) {
-    HBCI_Error_delete (err);
     return TRUE;
   }
   else {
-    printf("gnc_hbci_api_execute: Some message at executeQueue: %s",
-	   HBCI_Error_message (err));
-    HBCI_Error_delete (err);
+/*     printf("gnc_AB_BANKING_execute: Some message at executeQueue: %s", */
+/* 	   hbci_Error_message (err)); */
     GNCInteractor_show_nodelete (inter);
     return TRUE; /* <- This used to be a FALSE but this was probably
 		  * as wrong as it could get. @%$! */
@@ -448,6 +469,7 @@ static void *gnc_list_string_cb (const char *string, void *user_data)
   gchar **res = user_data;
   gchar *tmp1, *tmp2;
 
+  if (!string) return NULL;
   tmp1 = g_strdup (string);
   g_strstrip (tmp1);
 
@@ -469,23 +491,27 @@ static void *gnc_list_string_cb (const char *string, void *user_data)
 }
 
 
-char *gnc_hbci_descr_tognc (const HBCI_Transaction *h_trans)
+char *gnc_hbci_descr_tognc (const AB_TRANSACTION *h_trans)
 {
   /* Description */
   char *h_descr = NULL;
   char *othername = NULL;
   char *g_descr;
+  const GWEN_STRINGLIST *h_purpose = AB_Transaction_GetPurpose (h_trans);
+  const GWEN_STRINGLIST *h_remotename = AB_Transaction_GetRemoteName (h_trans);
 
   /* Don't use list_string_concat_delim here since we need to
      g_strstrip every single element of the string list, which is
      only done in our callback gnc_list_string_cb. The separator is
      also set there. */
-  list_string_foreach (HBCI_Transaction_description (h_trans), 
-		       &gnc_list_string_cb,
-		       &h_descr);
-  list_string_foreach (HBCI_Transaction_otherName (h_trans), 
-		       &gnc_list_string_cb,
-		       &othername);
+  if (h_purpose)
+    GWEN_StringList_ForEach (h_purpose,
+			     &gnc_list_string_cb,
+			     &h_descr);
+  if (h_remotename)
+    GWEN_StringList_ForEach (h_remotename,
+			     &gnc_list_string_cb,
+			     &othername);
   /*DEBUG("HBCI Description '%s'", h_descr);*/
 
   if (othername && (strlen (othername) > 0))
@@ -506,52 +532,57 @@ char *gnc_hbci_descr_tognc (const HBCI_Transaction *h_trans)
   return g_descr;
 }
 
-char *gnc_hbci_memo_tognc (const HBCI_Transaction *h_trans)
+char *gnc_hbci_memo_tognc (const AB_TRANSACTION *h_trans)
 {
   /* Memo in the Split. HBCI's transactionText contains strings like
    * "STANDING ORDER", "UEBERWEISUNGSGUTSCHRIFT", etc.  */
-  char *h_transactionText = 
-    g_strdup (HBCI_Transaction_transactionText (h_trans));
+  /*   char *h_transactionText =  */
+  /*     g_strdup (AB_TRANSACTION_transactionText (h_trans)); */
+  const char *h_remoteAccountNumber = 
+    AB_Transaction_GetRemoteAccountNumber (h_trans);
+  const char *h_remoteBankCode = 
+    AB_Transaction_GetRemoteBankCode (h_trans);
   char *h_otherAccountId =
-    g_strdup (HBCI_Transaction_otherAccountId (h_trans));
+    g_strdup (h_remoteAccountNumber ? h_remoteAccountNumber : _("unknown"));
   char *h_otherBankCode =
-    g_strdup (HBCI_Transaction_otherBankCode (h_trans));
+    g_strdup (h_remoteBankCode ? h_remoteBankCode : _("unknown"));
   char *g_memo;
 
-  g_strstrip (h_transactionText);
+  /*   g_strstrip (h_transactionText); */
   g_strstrip (h_otherAccountId);
   g_strstrip (h_otherBankCode);
 
   g_memo = 
-    (h_transactionText && (strlen(h_transactionText) > 0) ?
-     g_strdup_printf ("%s %s %s %s %s",
-		      h_transactionText,
+    (h_otherAccountId && (strlen (h_otherAccountId) > 0) ?
+     g_strdup_printf ("%s %s %s %s",
 		      _("Account"), h_otherAccountId,
 		      _("Bank"), h_otherBankCode) :
-     (h_otherAccountId && (strlen (h_otherAccountId) > 0) ?
-      g_strdup_printf ("%s %s %s %s",
-		       _("Account"), h_otherAccountId,
-		       _("Bank"), h_otherBankCode) :
-      g_strdup ("")));
+     g_strdup (""));
     
-  g_free (h_transactionText);
   g_free (h_otherAccountId);
   g_free (h_otherBankCode);
   return g_memo;
 }
 
 
+#if 0
 /** Return the only customer that can act on the specified account, or
     NULL if none was found. */
 const HBCI_Customer *
-gnc_hbci_get_first_customer(const HBCI_Account *h_acc)
+gnc_hbci_get_first_customer(const AB_ACCOUNT *h_acc)
 {
   /* Get one customer. */
-  const list_HBCI_Customer *custlist;
-    
-  custlist = HBCI_Account_authorizedCustomers (h_acc);
-  g_assert (custlist);
-  return choose_one_customer(gnc_ui_get_toplevel (), custlist);
+  const list_HBCI_User *userlist;
+  const HBCI_Bank *bank;
+  const HBCI_User *user;
+  g_assert(h_acc);
+  
+  bank = AB_ACCOUNT_bank (h_acc);
+  userlist = HBCI_Bank_users (bank);
+  g_assert (userlist);
+  user = choose_one_user(gnc_ui_get_toplevel (), userlist);
+  g_assert (user);
+  return choose_one_customer(gnc_ui_get_toplevel (), HBCI_User_customers (user));
 }
 
 const char *bank_to_str (const HBCI_Bank *bank)
@@ -678,7 +709,7 @@ choose_one_customer (gncUIWidget parent, const list_HBCI_Customer *custlist)
 			 * is the name of the bank. %s is the bank
 			 * code. */
 			g_strdup_printf (_("%s (%s) at bank %s (%s)"),
-					 HBCI_Customer_custName (customer),
+					 HBCI_Customer_name (customer),
 					 HBCI_Customer_custId (customer),
 					 bank_to_str (HBCI_User_bank(HBCI_Customer_user(customer))),
 					 HBCI_Bank_bankCode (HBCI_User_bank(HBCI_Customer_user(customer)))));
@@ -754,7 +785,7 @@ choose_one_user (gncUIWidget parent, const list_HBCI_User *userlist)
 	radio_list = g_list_append
 	    (radio_list, 
 	     g_strdup_printf (_("%s (%s) at bank %s (%s)"),
-			      HBCI_User_userName (user),
+			      HBCI_User_name (user),
 			      HBCI_User_userId (user),
 			      bank_to_str (HBCI_User_bank(user)),
 			      HBCI_Bank_bankCode (HBCI_User_bank(user))));
@@ -792,3 +823,14 @@ choose_one_user (gncUIWidget parent, const list_HBCI_User *userlist)
   g_assert_not_reached();
   return NULL;
 }
+#endif
+
+char *gnc_AB_VALUE_toReadableString(const AB_VALUE *v)
+{
+  char tmp[100];
+  if (v)
+    sprintf(tmp, "%.2f %s", AB_Value_GetValue(v), AB_Value_GetCurrency(v));
+  else
+    sprintf(tmp, "%.2f", 0.0);
+  return g_strdup(tmp);
+}
