@@ -4,7 +4,7 @@
 ;;;  form. 
 ;;;
 ;;;  Bill Gribble <grib@billgribble.com> 20 Feb 2000 
-;;;  $Id: qif-dialog-utils.scm,v 1.3 2002/07/09 19:53:30 warlord Exp $
+;;;  $Id: qif-dialog-utils.scm,v 1.3.2.1 2004/08/19 19:59:00 warlord Exp $
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (use-modules (ice-9 regex))
@@ -19,8 +19,11 @@
 
 (define (default-interest-acct brokerage security) 
   (string-append (_ "Interest") (gnc:account-separator-char) 
-                 brokerage (gnc:account-separator-char)  
-                 security))
+                 brokerage
+		 (if (string=? security "")
+		  ""
+		  (string-append (gnc:account-separator-char)  
+				  security))))
 
 (define (default-capital-return-acct brokerage security) 
   (string-append (_ "Cap Return") (gnc:account-separator-char) 
