@@ -42,21 +42,30 @@
 (define gnc:current-config-auto
   (build-path (getenv "HOME") ".gnucash" "config-1.8.auto"))
 
+(define gnc:current-saved-reports
+  (build-path (getenv "HOME") ".gnucash" "saved-reports-1.8"))
+
 (define gnc:load-user-config-if-needed
   (let ((user-config-loaded? #f))
 
-    (define (try-load file-suffix)
+    (define (try-load-no-set file-suffix)
       (let ((file (build-path (getenv "HOME") ".gnucash" file-suffix)))
+	(gnc:debug "trying to load " file)
         (if (access? file F_OK)
             (if (false-if-exception (primitive-load file))
-                (begin
-                  (set! user-config-loaded? #t)
-                  #t)
+		#t
                 (begin
                   (gnc:warn "failure loading " file)
                   #f))
             #f)))
 
+    (define (try-load file-suffix)
+      (if (try-load-no-set file-suffix)
+	  (begin
+	    (set! user-config-loaded? #t)
+	    #t)
+	  #f))
+
     (lambda ()
       (if (not user-config-loaded?)
           (begin
@@ -66,7 +75,11 @@
 		    ;; rolls around bump the 1.4 (unnumbered) files
 		    ;; off the list.
                     '("config-1.8.user" "config-1.6.user" "config.user"
-                      "config-1.8.auto" "config-1.6.auto" "config.auto")))))))
+                      "config-1.8.auto" "config-1.6.auto" "config.auto"))
+	    (gnc:debug "loading saved reports")
+	    (or-map try-load-no-set
+		    '("saved-reports-1.8"))
+	    )))))
 
 ;; the system config should probably be loaded from some directory
 ;; that wouldn't be a site wide mounted directory, like /usr/share
